@@ -33,7 +33,8 @@ seq_to_tclist(PyObject *pyvalues)
     PyObject *pyseq;
     Py_ssize_t len, i;
     TCLIST *values;
-    char *value;
+    void *value;
+    int value_size;
 
     if (PyBytes_Check(pyvalues) || PyUnicode_Check(pyvalues)) {
         set_error(PyExc_TypeError, msg);
@@ -44,6 +45,10 @@ seq_to_tclist(PyObject *pyvalues)
         return NULL;
     }
     len = PySequence_Fast_GET_SIZE(pyseq);
+    if (check_py_ssize_t_len(len, pyseq)) {
+        Py_DECREF(pyseq);
+        return NULL;
+    }
     values = tclistnew2((int)len);
     if (!values) {
         set_error(Error, "could not create TCLIST, memory issue?");
@@ -51,13 +56,13 @@ seq_to_tclist(PyObject *pyvalues)
         return NULL;
     }
     for (i = 0; i < len; i++) {
-        value = PyBytes_AsString(PySequence_Fast_GET_ITEM(pyseq, i));
-        if (!value) {
+        if (bytes_to_void(PySequence_Fast_GET_ITEM(pyseq, i), &value,
+                          &value_size)) {
             Py_DECREF(pyseq);
             tclistdel(values);
             return NULL;
         }
-        tclistpush2(values, value);
+        tclistpush(values, value, value_size);
     }
     Py_DECREF(pyseq);
     return values;
@@ -162,17 +167,17 @@ is positioned to the next available record.");
 static PyObject *
 BDBCursor_jump(BDBCursor *self, PyObject *args)
 {
-    const char *key;
+    void *key;
+    int key_size;
     PyObject *pykey;
 
     if (!PyArg_ParseTuple(args, "O:jump", &pykey)) {
         return NULL;
     }
-    key = PyBytes_AsString(pykey);
-    if (!key) {
+    if (bytes_to_void(pykey, &key, &key_size)) {
         return NULL;
     }
-    if (!tcbdbcurjump2(self->cur, key)) {
+    if (!tcbdbcurjump(self->cur, key, key_size)) {
         return set_bdbcursor_move_error(self->bdb->bdb);
     }
     Py_RETURN_NONE;
@@ -224,18 +229,18 @@ Store a value at/around the cursor's current position.\n\
 static PyObject *
 BDBCursor_put(BDBCursor *self, PyObject *args)
 {
-    const char *value;
+    void *value;
+    int value_size;
     PyObject *pyvalue;
     int mode = BDBCPCURRENT;
 
     if (!PyArg_ParseTuple(args, "O|i:put", &pyvalue, &mode)) {
         return NULL;
     }
-    value = PyBytes_AsString(pyvalue);
-    if (!value) {
+    if (bytes_to_void(pyvalue, &value, &value_size)) {
         return NULL;
     }
-    if (!tcbdbcurput2(self->cur, value, mode)) {
+    if (!tcbdbcurput(self->cur, value, value_size, mode)) {
         return set_bdb_error(self->bdb->bdb, NULL);
     }
     self->bdb->changed = true;
@@ -270,15 +275,16 @@ Get current key.");
 static PyObject *
 BDBCursor_key(BDBCursor *self)
 {
-    const char *key;
+    void *key;
+    int key_size;
     PyObject *pykey;
 
-    key = tcbdbcurkey2(self->cur);
+    key = tcbdbcurkey(self->cur, &key_size);
     if (!key) {
         return set_bdb_error(self->bdb->bdb, NULL);
     }
-    pykey = PyBytes_FromString(key);
-    tcfree((void *)key);
+    pykey = void_to_bytes(key, key_size);
+    tcfree(key);
     return pykey;
 }
 
@@ -292,15 +298,16 @@ Get current value.");
 static PyObject *
 BDBCursor_value(BDBCursor *self)
 {
-    const char *value;
+    void *value;
+    int value_size;
     PyObject *pyvalue;
 
-    value = tcbdbcurval2(self->cur);
+    value = tcbdbcurval(self->cur, &value_size);
     if (!value) {
         return set_bdb_error(self->bdb->bdb, NULL);
     }
-    pyvalue = PyBytes_FromString(value);
-    tcfree((void *)value);
+    pyvalue = void_to_bytes(value, value_size);
+    tcfree(value);
     return pyvalue;
 }
 
@@ -315,22 +322,21 @@ static PyObject *
 BDBCursor_item(BDBCursor *self)
 {
     TCXSTR *key, *value;
-    PyObject *pyresult;
-#if PY_MAJOR_VERSION >= 3
-    const char *format = "(yy)";
-#else
-    const char *format = "(ss)";
-#endif
+    PyObject *pykey, *pyvalue, *pyresult = NULL;
 
     key = tcxstrnew();
     value = tcxstrnew();
     if (!tcbdbcurrec(self->cur, key, value)) {
         set_bdb_error(self->bdb->bdb, NULL);
-        pyresult = NULL;
     }
     else {
-        pyresult = Py_BuildValue(format, (char *)tcxstrptr(key),
-                                 (char *)tcxstrptr(value));
+        pykey = tcxstr_to_bytes(key);
+        pyvalue = tcxstr_to_bytes(value);
+        if (pykey && pyvalue) {
+            pyresult = PyTuple_Pack(2, pykey, pyvalue);
+        }
+        Py_XDECREF(pykey);
+        Py_XDECREF(pyvalue);
     }
     tcxstrdel(key);
     tcxstrdel(value);
@@ -387,6 +393,226 @@ static PyTypeObject BDBCursorType = {
 };
 
 
+/*******************************************************************************
+* BDB iterator types
+*******************************************************************************/
+
+/* new_BDBIter */
+static PyObject *
+new_BDBIter(BDB *self, PyTypeObject *type)
+{
+    PyObject *iter = DBIter_tp_new(type, (PyObject *)self);
+    if (!iter) {
+        return NULL;
+    }
+    if (!tcbdbcurfirst(self->cur)) {
+        if (tcbdbecode(self->bdb) != TCENOREC) {
+            Py_DECREF(iter);
+            return set_bdb_error(self->bdb, NULL);
+        }
+    }
+    self->changed = false;
+    return iter;
+}
+
+
+/* BDBIterKeysType.tp_iternext */
+static PyObject *
+BDBIterKeys_tp_iternext(DBIter *self)
+{
+    BDB *bdb = (BDB *)self->db;
+    void *key;
+    int key_size;
+    PyObject *pykey;
+
+    if (bdb->changed) {
+        return set_error(Error, "BDB changed during iteration");
+    }
+    key = tcbdbcurkey(bdb->cur, &key_size);
+    if (!key) {
+        if (tcbdbecode(bdb->bdb) == TCENOREC) {
+            return set_stopiteration_error();
+        }
+        return set_bdb_error(bdb->bdb, NULL);
+    }
+    pykey = void_to_bytes(key, key_size);
+    tcfree(key);
+    if (!pykey) {
+        return NULL;
+    }
+    tcbdbcurnext(bdb->cur);
+    return pykey;
+}
+
+
+/* BDBIterKeysType */
+static PyTypeObject BDBIterKeysType = {
+    PyVarObject_HEAD_INIT(NULL, 0)
+    "tokyo.cabinet.BDBIterKeys",              /*tp_name*/
+    sizeof(DBIter),                           /*tp_basicsize*/
+    0,                                        /*tp_itemsize*/
+    (destructor)DBIter_tp_dealloc,            /*tp_dealloc*/
+    0,                                        /*tp_print*/
+    0,                                        /*tp_getattr*/
+    0,                                        /*tp_setattr*/
+    0,                                        /*tp_compare*/
+    0,                                        /*tp_repr*/
+    0,                                        /*tp_as_number*/
+    0,                                        /*tp_as_sequence*/
+    0,                                        /*tp_as_mapping*/
+    0,                                        /*tp_hash */
+    0,                                        /*tp_call*/
+    0,                                        /*tp_str*/
+    0,                                        /*tp_getattro*/
+    0,                                        /*tp_setattro*/
+    0,                                        /*tp_as_buffer*/
+    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_HAVE_GC,  /*tp_flags*/
+    0,                                        /*tp_doc*/
+    (traverseproc)DBIter_tp_traverse,         /*tp_traverse*/
+    (inquiry)DBIter_tp_clear,                 /*tp_clear*/
+    0,                                        /*tp_richcompare*/
+    0,                                        /*tp_weaklistoffset*/
+    PyObject_SelfIter,                        /*tp_iter*/
+    (iternextfunc)BDBIterKeys_tp_iternext,    /*tp_iternext*/
+    DBIter_tp_methods,                        /*tp_methods*/
+};
+
+
+/* BDBIterValuesType.tp_iternext */
+static PyObject *
+BDBIterValues_tp_iternext(DBIter *self)
+{
+    BDB *bdb = (BDB *)self->db;
+    void *value;
+    int value_size;
+    PyObject *pyvalue;
+
+    if (bdb->changed) {
+        return set_error(Error, "BDB changed during iteration");
+    }
+    value = tcbdbcurval(bdb->cur, &value_size);
+    if (!value) {
+        if (tcbdbecode(bdb->bdb) == TCENOREC) {
+            return set_stopiteration_error();
+        }
+        return set_bdb_error(bdb->bdb, NULL);
+    }
+    pyvalue = void_to_bytes(value, value_size);
+    tcfree(value);
+    if (!pyvalue) {
+        return NULL;
+    }
+    tcbdbcurnext(bdb->cur);
+    return pyvalue;
+}
+
+
+/* BDBIterValuesType */
+static PyTypeObject BDBIterValuesType = {
+    PyVarObject_HEAD_INIT(NULL, 0)
+    "tokyo.cabinet.BDBIterValues",            /*tp_name*/
+    sizeof(DBIter),                           /*tp_basicsize*/
+    0,                                        /*tp_itemsize*/
+    (destructor)DBIter_tp_dealloc,            /*tp_dealloc*/
+    0,                                        /*tp_print*/
+    0,                                        /*tp_getattr*/
+    0,                                        /*tp_setattr*/
+    0,                                        /*tp_compare*/
+    0,                                        /*tp_repr*/
+    0,                                        /*tp_as_number*/
+    0,                                        /*tp_as_sequence*/
+    0,                                        /*tp_as_mapping*/
+    0,                                        /*tp_hash */
+    0,                                        /*tp_call*/
+    0,                                        /*tp_str*/
+    0,                                        /*tp_getattro*/
+    0,                                        /*tp_setattro*/
+    0,                                        /*tp_as_buffer*/
+    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_HAVE_GC,  /*tp_flags*/
+    0,                                        /*tp_doc*/
+    (traverseproc)DBIter_tp_traverse,         /*tp_traverse*/
+    (inquiry)DBIter_tp_clear,                 /*tp_clear*/
+    0,                                        /*tp_richcompare*/
+    0,                                        /*tp_weaklistoffset*/
+    PyObject_SelfIter,                        /*tp_iter*/
+    (iternextfunc)BDBIterValues_tp_iternext,  /*tp_iternext*/
+    DBIter_tp_methods,                        /*tp_methods*/
+};
+
+
+/* BDBIterItemsType.tp_iternext */
+static PyObject *
+BDBIterItems_tp_iternext(DBIter *self)
+{
+    BDB *bdb = (BDB *)self->db;
+    TCXSTR *key, *value;
+    PyObject *pykey, *pyvalue, *pyresult = NULL;
+
+    if (bdb->changed) {
+        return set_error(Error, "BDB changed during iteration");
+    }
+    key = tcxstrnew();
+    value = tcxstrnew();
+    if (!tcbdbcurrec(bdb->cur, key, value)) {
+        if (tcbdbecode(bdb->bdb) == TCENOREC) {
+            set_stopiteration_error();
+        }
+        else {
+            set_bdb_error(bdb->bdb, NULL);
+        }
+    }
+    else {
+        pykey = tcxstr_to_bytes(key);
+        pyvalue = tcxstr_to_bytes(value);
+        if (pykey && pyvalue) {
+            pyresult = PyTuple_Pack(2, pykey, pyvalue);
+        }
+        Py_XDECREF(pykey);
+        Py_XDECREF(pyvalue);
+    }
+    tcxstrdel(key);
+    tcxstrdel(value);
+    if (!pyresult) {
+        return NULL;
+    }
+    tcbdbcurnext(bdb->cur);
+    return pyresult;
+}
+
+
+/* BDBIterItemsType */
+static PyTypeObject BDBIterItemsType = {
+    PyVarObject_HEAD_INIT(NULL, 0)
+    "tokyo.cabinet.BDBIterItems",             /*tp_name*/
+    sizeof(DBIter),                           /*tp_basicsize*/
+    0,                                        /*tp_itemsize*/
+    (destructor)DBIter_tp_dealloc,            /*tp_dealloc*/
+    0,                                        /*tp_print*/
+    0,                                        /*tp_getattr*/
+    0,                                        /*tp_setattr*/
+    0,                                        /*tp_compare*/
+    0,                                        /*tp_repr*/
+    0,                                        /*tp_as_number*/
+    0,                                        /*tp_as_sequence*/
+    0,                                        /*tp_as_mapping*/
+    0,                                        /*tp_hash */
+    0,                                        /*tp_call*/
+    0,                                        /*tp_str*/
+    0,                                        /*tp_getattro*/
+    0,                                        /*tp_setattro*/
+    0,                                        /*tp_as_buffer*/
+    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_HAVE_GC,  /*tp_flags*/
+    0,                                        /*tp_doc*/
+    (traverseproc)DBIter_tp_traverse,         /*tp_traverse*/
+    (inquiry)DBIter_tp_clear,                 /*tp_clear*/
+    0,                                        /*tp_richcompare*/
+    0,                                        /*tp_weaklistoffset*/
+    PyObject_SelfIter,                        /*tp_iter*/
+    (iternextfunc)BDBIterItems_tp_iternext,   /*tp_iternext*/
+    DBIter_tp_methods,                        /*tp_methods*/
+};
+
+
 /*******************************************************************************
 * BDBType
 *******************************************************************************/
@@ -408,9 +634,9 @@ BDB_cmp_cb(const char *a, int a_size, const char *b, int b_size, void *op)
         goto fail;
     }
 #if PY_MAJOR_VERSION >= 3
-    if(PyLong_Check(pyresult)) {
+    if (PyLong_Check(pyresult)) {
 #else
-    if(PyInt_Check(pyresult)) {
+    if (PyInt_Check(pyresult)) {
 #endif
         result = (int)PyLong_AsLong(pyresult);
         if (result == -1 && PyErr_Occurred()) {
@@ -492,13 +718,13 @@ BDB_tp_new(PyTypeObject *type, PyObject *args, PyObject *kwargs)
 static
 int BDB_Contains(BDB *self, PyObject *pykey)
 {
-    const char *key, *value;
+    void *key, *value;
+    int key_size, value_size;
 
-    key = PyBytes_AsString(pykey);
-    if (!key) {
+    if (bytes_to_void(pykey, &key, &key_size)) {
         return -1;
     }
-    value = tcbdbget2(self->bdb, key);
+    value = tcbdbget(self->bdb, key, key_size, &value_size);
     if (!value) {
         if (tcbdbecode(self->bdb) == TCENOREC) {
             return 0;
@@ -506,7 +732,7 @@ int BDB_Contains(BDB *self, PyObject *pykey)
         set_bdb_error(self->bdb, NULL);
         return -1;
     }
-    tcfree((void *)value);
+    tcfree(value);
     return 1;
 }
 
@@ -528,7 +754,7 @@ static PySequenceMethods BDB_tp_as_sequence = {
 static Py_ssize_t
 BDB_Length(BDB *self)
 {
-    return (Py_ssize_t)tcbdbrnum(self->bdb);
+    return DB_Length(tcbdbrnum(self->bdb));
 }
 
 
@@ -536,19 +762,19 @@ BDB_Length(BDB *self)
 static PyObject *
 BDB_GetItem(BDB *self, PyObject *pykey)
 {
-    const char *key, *value;
+    void *key, *value;
+    int key_size, value_size;
     PyObject *pyvalue;
 
-    key = PyBytes_AsString(pykey);
-    if (!key) {
+    if (bytes_to_void(pykey, &key, &key_size)) {
         return NULL;
     }
-    value = tcbdbget2(self->bdb, key);
+    value = tcbdbget(self->bdb, key, key_size, &value_size);
     if (!value) {
         return set_bdb_error(self->bdb, key);
     }
-    pyvalue = PyBytes_FromString(value);
-    tcfree((void *)value);
+    pyvalue = void_to_bytes(value, value_size);
+    tcfree(value);
     return pyvalue;
 }
 
@@ -557,24 +783,23 @@ BDB_GetItem(BDB *self, PyObject *pykey)
 static int
 BDB_SetItem(BDB *self, PyObject *pykey, PyObject *pyvalue)
 {
-    const char *key, *value;
+    void *key, *value;
+    int key_size, value_size;
 
-    key = PyBytes_AsString(pykey);
-    if (!key) {
+    if (bytes_to_void(pykey, &key, &key_size)) {
         return -1;
     }
     if (pyvalue) {
-        value = PyBytes_AsString(pyvalue);
-        if (!value) {
+        if (bytes_to_void(pyvalue, &value, &value_size)) {
             return -1;
         }
-        if (!tcbdbput2(self->bdb, key, value)) {
+        if (!tcbdbput(self->bdb, key, key_size, value, value_size)) {
             set_bdb_error(self->bdb, NULL);
             return -1;
         }
     }
     else {
-        if (!tcbdbout2(self->bdb, key)) {
+        if (!tcbdbout(self->bdb, key, key_size)) {
             set_bdb_error(self->bdb, key);
             return -1;
         }
@@ -596,49 +821,7 @@ static PyMappingMethods BDB_tp_as_mapping = {
 static PyObject *
 BDB_tp_iter(BDB *self)
 {
-    if (!tcbdbcurfirst(self->cur)) {
-        if (tcbdbecode(self->bdb) != TCENOREC) {
-            return set_bdb_error(self->bdb, NULL);
-        }
-    }
-    self->changed = false;
-    Py_INCREF(self);
-    return (PyObject *)self;
-}
-
-
-/* BDBType.tp_iternext */
-static PyObject *
-BDB_tp_iternext(BDB *self)
-{
-    const char *key;
-    PyObject *pykey;
-
-    if (self->changed) {
-        return set_error(Error, "BDB changed during iteration");
-    }
-    key = tcbdbcurkey2(self->cur);
-    if (!key) {
-        if (tcbdbecode(self->bdb) == TCENOREC) {
-            return set_stopiteration_error();
-        }
-        return set_bdb_error(self->bdb, NULL);
-    }
-    pykey = PyBytes_FromString(key);
-    tcfree((void *)key);
-    tcbdbcurnext(self->cur);
-    return pykey;
-}
-
-
-/* BDB.__length_hint__ */
-PyDoc_STRVAR(BDB_length_hint_doc,
-"Private method returning an estimate of len(list(bdb)).");
-
-static PyObject *
-BDB_length_hint(BDB *self)
-{
-    return PyLong_FromSsize_t(BDB_Length(self));
+    return new_BDBIter(self, &BDBIterKeysType);
 }
 
 
@@ -787,8 +970,8 @@ static PyObject *
 BDB_get(BDB *self, PyObject *args, PyObject *kwargs)
 {
     PyObject *pykey, *pyresult, *duplicate = Py_False;
-    char *key;
-    Py_ssize_t key_size;
+    void *key;
+    int key_size;
     TCLIST *result;
 
     static char *kwlist[] = {"key", "duplicate", NULL};
@@ -803,11 +986,11 @@ BDB_get(BDB *self, PyObject *args, PyObject *kwargs)
     if (duplicate == Py_False) {
         return BDB_GetItem(self, pykey);
     }
-    if(PyBytes_AsStringAndSize(pykey, &key, &key_size)) {
+    if (bytes_to_void(pykey, &key, &key_size)) {
         return NULL;
     }
     Py_BEGIN_ALLOW_THREADS
-    result = tcbdbget4(self->bdb, key, (int)key_size);
+    result = tcbdbget4(self->bdb, key, key_size);
     Py_END_ALLOW_THREADS
     if (!result) {
         return set_bdb_error(self->bdb, key);
@@ -828,8 +1011,8 @@ static PyObject *
 BDB_remove(BDB *self, PyObject *args, PyObject *kwargs)
 {
     PyObject *pykey, *duplicate = Py_False;
-    char *key;
-    Py_ssize_t key_size;
+    void *key;
+    int key_size;
     bool result;
 
     static char *kwlist[] = {"key", "duplicate", NULL};
@@ -847,11 +1030,11 @@ BDB_remove(BDB *self, PyObject *args, PyObject *kwargs)
         }
         Py_RETURN_NONE;
     }
-    if(PyBytes_AsStringAndSize(pykey, &key, &key_size)) {
+    if (bytes_to_void(pykey, &key, &key_size)) {
         return NULL;
     }
     Py_BEGIN_ALLOW_THREADS
-    result = tcbdbout3(self->bdb, key, (int)key_size);
+    result = tcbdbout3(self->bdb, key, key_size);
     Py_END_ALLOW_THREADS
     if (!result) {
         return set_bdb_error(self->bdb, key);
@@ -871,7 +1054,8 @@ static PyObject *
 BDB_put(BDB *self, PyObject *args, PyObject *kwargs)
 {
     PyObject *pykey, *pyvalue, *duplicate = Py_False;
-    char *key, *value;
+    void *key, *value;
+    int key_size, value_size;
 
     static char *kwlist[] = {"key", "value", "duplicate", NULL};
 
@@ -888,12 +1072,11 @@ BDB_put(BDB *self, PyObject *args, PyObject *kwargs)
         }
         Py_RETURN_NONE;
     }
-    key = PyBytes_AsString(pykey);
-    value = PyBytes_AsString(pyvalue);
-    if (!(key && value)) {
+    if (bytes_to_void(pykey, &key, &key_size) ||
+        bytes_to_void(pyvalue, &value, &value_size)) {
         return NULL;
     }
-    if (!tcbdbputdup2(self->bdb, key, value)) {
+    if (!tcbdbputdup(self->bdb, key, key_size, value, value_size)) {
         return set_bdb_error(self->bdb, NULL);
     }
     self->changed = true;
@@ -911,18 +1094,18 @@ put), this method raises KeyError if key is already in the database.");
 static PyObject *
 BDB_putkeep(BDB *self, PyObject *args)
 {
-    const char *key, *value;
+    void *key, *value;
+    int key_size, value_size;
     PyObject *pykey, *pyvalue;
 
     if (!PyArg_ParseTuple(args, "OO:putkeep", &pykey, &pyvalue)) {
         return NULL;
     }
-    key = PyBytes_AsString(pykey);
-    value = PyBytes_AsString(pyvalue);
-    if (!(key && value)) {
+    if (bytes_to_void(pykey, &key, &key_size) ||
+        bytes_to_void(pyvalue, &value, &value_size)) {
         return NULL;
     }
-    if (!tcbdbputkeep2(self->bdb, key, value)) {
+    if (!tcbdbputkeep(self->bdb, key, key_size, value, value_size)) {
         return set_bdb_error(self->bdb, key);
     }
     self->changed = true;
@@ -940,18 +1123,18 @@ If there is no corresponding record, a new record is stored.");
 static PyObject *
 BDB_putcat(BDB *self, PyObject *args)
 {
-    const char *key, *value;
+    void *key, *value;
+    int key_size, value_size;
     PyObject *pykey, *pyvalue;
 
     if (!PyArg_ParseTuple(args, "OO:putcat", &pykey, &pyvalue)) {
         return NULL;
     }
-    key = PyBytes_AsString(pykey);
-    value = PyBytes_AsString(pyvalue);
-    if (!(key && value)) {
+    if (bytes_to_void(pykey, &key, &key_size) ||
+        bytes_to_void(pyvalue, &value, &value_size)) {
         return NULL;
     }
-    if (!tcbdbputcat2(self->bdb, key, value)) {
+    if (!tcbdbputcat(self->bdb, key, key_size, value, value_size)) {
         return set_bdb_error(self->bdb, NULL);
     }
     self->changed = true;
@@ -968,8 +1151,8 @@ PyDoc_STRVAR(BDB_putdup_doc,
 static PyObject *
 BDB_putdup(BDB *self, PyObject *args)
 {
-    char *key;
-    Py_ssize_t key_size;
+    void *key;
+    int key_size;
     TCLIST *values;
     PyObject *pykey, *pyvalues;
     bool result;
@@ -977,7 +1160,7 @@ BDB_putdup(BDB *self, PyObject *args)
     if (!PyArg_ParseTuple(args, "OO:putdup", &pykey, &pyvalues)) {
         return NULL;
     }
-    if(PyBytes_AsStringAndSize(pykey, &key, &key_size)) {
+    if (bytes_to_void(pykey, &key, &key_size)) {
         return NULL;
     }
     values = seq_to_tclist(pyvalues);
@@ -985,13 +1168,12 @@ BDB_putdup(BDB *self, PyObject *args)
         return NULL;
     }
     Py_BEGIN_ALLOW_THREADS
-    result = tcbdbputdup3(self->bdb, key, (int)key_size, values);
+    result = tcbdbputdup3(self->bdb, key, key_size, values);
     Py_END_ALLOW_THREADS
+    tclistdel(values);
     if (!result) {
-        tclistdel(values);
         return set_bdb_error(self->bdb, NULL);
     }
-    tclistdel(values);
     self->changed = true;
     Py_RETURN_NONE;
 }
@@ -1024,20 +1206,19 @@ is applied.");
 static PyObject *
 BDB_searchkeys(BDB *self, PyObject *args)
 {
-    const char *prefix;
-    int max = -1;
+    void *prefix;
+    int prefix_size, max = -1;
     TCLIST *result;
     PyObject *pyprefix, *pyresult;
 
     if (!PyArg_ParseTuple(args, "O|i:searchkeys", &pyprefix, &max)) {
         return NULL;
     }
-    prefix = PyBytes_AsString(pyprefix);
-    if (!prefix) {
+    if (bytes_to_void(pyprefix, &prefix, &prefix_size)) {
         return NULL;
     }
     Py_BEGIN_ALLOW_THREADS
-    result = tcbdbfwmkeys2(self->bdb, prefix, max);
+    result = tcbdbfwmkeys(self->bdb, prefix, prefix_size, max);
     Py_END_ALLOW_THREADS
     pyresult = tclist_to_frozenset(result);
     tclistdel(result);
@@ -1054,8 +1235,8 @@ PyDoc_STRVAR(BDB_range_doc,
 static PyObject *
 BDB_range(BDB *self, PyObject *args, PyObject *kwargs)
 {
-    const char *begin = NULL, *end = NULL;
-    int max = -1;
+    void *begin = NULL, *end = NULL;
+    int begin_size, end_size, max = -1;
     TCLIST *result;
     PyObject *pybegin = Py_None, *pyend = Py_None, *pyresult;
 
@@ -1066,19 +1247,18 @@ BDB_range(BDB *self, PyObject *args, PyObject *kwargs)
         return NULL;
     }
     if (pybegin != Py_None) {
-        begin = PyBytes_AsString(pybegin);
-        if (!begin) {
+        if (bytes_to_void(pybegin, &begin, &begin_size)) {
             return NULL;
         }
     }
     if (pyend != Py_None) {
-        end = PyBytes_AsString(pyend);
-        if (!end) {
+        if (bytes_to_void(pyend, &end, &end_size)) {
             return NULL;
         }
     }
     Py_BEGIN_ALLOW_THREADS
-    result = tcbdbrange2(self->bdb, begin, 1, end, 1, max);
+    result = tcbdbrange(self->bdb, begin, begin_size, true,
+                        end, end_size, true, max);
     Py_END_ALLOW_THREADS
     pyresult = tclist_to_frozenset(result);
     tclistdel(result);
@@ -1286,9 +1466,9 @@ BDB_setcmpfunc(BDB *self, PyObject *args)
         return NULL;
     }
 #if PY_MAJOR_VERSION >= 3
-    if(PyLong_Check(pycb)) {
+    if (PyLong_Check(pycb)) {
 #else
-    if(PyInt_Check(pycb)) {
+    if (PyInt_Check(pycb)) {
 #endif
         cmp = (int)PyLong_AsLong(pycb);
         if (cmp == -1 && PyErr_Occurred()) {
@@ -1342,25 +1522,26 @@ static PyObject *
 BDB_addint(BDB *self, PyObject *args)
 {
     PyObject *pykey;
-    char *key;
-    Py_ssize_t key_size;
-    int num, result, ecode;
+    void *key;
+    int key_size, num, result, ecode;
 
     if (!PyArg_ParseTuple(args, "Oi:addint", &pykey, &num)) {
         return NULL;
     }
-    if(PyBytes_AsStringAndSize(pykey, &key, &key_size)) {
+    if (bytes_to_void(pykey, &key, &key_size)) {
         return NULL;
     }
-    result = tcbdbaddint(self->bdb, key, (int)key_size, num);
+    result = tcbdbaddint(self->bdb, key, key_size, num);
     if (result == INT_MIN) {
         ecode = tcbdbecode(self->bdb);
         if (ecode != TCESUCCESS && ecode != TCENOREC) {
             return set_bdb_error(self->bdb, key);
         }
     }
-    self->changed = true;
-    return Py_BuildValue("i", result);
+    if (num) {
+        self->changed = true;
+    }
+    return PyInt_FromLong((long)result);
 }
 
 
@@ -1381,29 +1562,68 @@ static PyObject *
 BDB_adddouble(BDB *self, PyObject *args)
 {
     PyObject *pykey;
-    char *key;
-    Py_ssize_t key_size;
+    void *key;
+    int key_size;
     double num, result;
 
     if (!PyArg_ParseTuple(args, "Od:adddouble", &pykey, &num)) {
         return NULL;
     }
-    if(PyBytes_AsStringAndSize(pykey, &key, &key_size)) {
+    if (bytes_to_void(pykey, &key, &key_size)) {
         return NULL;
     }
-    result = tcbdbadddouble(self->bdb, key, (int)key_size, num);
+    result = tcbdbadddouble(self->bdb, key, key_size, num);
     if (Py_IS_NAN(result)) {
         return set_bdb_error(self->bdb, key);
     }
-    self->changed = true;
+    if (num) {
+        self->changed = true;
+    }
     return PyFloat_FromDouble(result);
 }
 
 
+/* BDB.iterkeys() */
+PyDoc_STRVAR(BDB_iterkeys_doc,
+"iterkeys()\n\
+\n\
+Return an iterator over the database's keys.");
+
+static PyObject *
+BDB_iterkeys(BDB *self)
+{
+    return new_BDBIter(self, &BDBIterKeysType);
+}
+
+
+/* BDB.itervalues() */
+PyDoc_STRVAR(BDB_itervalues_doc,
+"itervalues()\n\
+\n\
+Return an iterator over the database's values.");
+
+static PyObject *
+BDB_itervalues(BDB *self)
+{
+    return new_BDBIter(self, &BDBIterValuesType);
+}
+
+
+/* BDB.iteritems() */
+PyDoc_STRVAR(BDB_iteritems_doc,
+"iteritems()\n\
+\n\
+Return an iterator over the database's items.");
+
+static PyObject *
+BDB_iteritems(BDB *self)
+{
+    return new_BDBIter(self, &BDBIterItemsType);
+}
+
+
 /* BDBType.tp_methods */
 static PyMethodDef BDB_tp_methods[] = {
-    {"__length_hint__", (PyCFunction)BDB_length_hint, METH_NOARGS,
-     BDB_length_hint_doc},
     {"open", (PyCFunction)BDB_open, METH_VARARGS, BDB_open_doc},
     {"close", (PyCFunction)BDB_close, METH_NOARGS, BDB_close_doc},
     {"clear", (PyCFunction)BDB_clear, METH_NOARGS, BDB_clear_doc},
@@ -1433,6 +1653,9 @@ static PyMethodDef BDB_tp_methods[] = {
     {"setcmpfunc", (PyCFunction)BDB_setcmpfunc, METH_VARARGS, BDB_setcmpfunc_doc},
     {"addint", (PyCFunction)BDB_addint, METH_VARARGS, BDB_addint_doc},
     {"adddouble", (PyCFunction)BDB_adddouble, METH_VARARGS, BDB_adddouble_doc},
+    {"iterkeys", (PyCFunction)BDB_iterkeys, METH_NOARGS, BDB_iterkeys_doc},
+    {"itervalues", (PyCFunction)BDB_itervalues, METH_NOARGS, BDB_itervalues_doc},
+    {"iteritems", (PyCFunction)BDB_iteritems, METH_NOARGS, BDB_iteritems_doc},
     {NULL}  /* Sentinel */
 };
 
@@ -1448,7 +1671,7 @@ BDB_path_get(BDB *self, void *closure)
 
     path = tcbdbpath(self->bdb);
     if (path) {
-        return Py_BuildValue("s", path);
+        return PyString_FromString(path);
     }
     Py_RETURN_NONE;
 }
@@ -1501,7 +1724,7 @@ static PyTypeObject BDBType = {
     0,                                        /*tp_richcompare*/
     0,                                        /*tp_weaklistoffset*/
     (getiterfunc)BDB_tp_iter,                 /*tp_iter*/
-    (iternextfunc)BDB_tp_iternext,            /*tp_iternext*/
+    0,                                        /*tp_iternext*/
     BDB_tp_methods,                           /*tp_methods*/
     0,                                        /*tp_members*/
     BDB_tp_getsets,                           /*tp_getsets*/
