@@ -15,6 +15,217 @@ set_hdb_error(TCHDB *hdb, const char *key)
 }
 
 
+/*******************************************************************************
+* HDB iterator types
+*******************************************************************************/
+
+/* new_HDBIter */
+static PyObject *
+new_HDBIter(HDB *self, PyTypeObject *type)
+{
+    PyObject *iter = DBIter_tp_new(type, (PyObject *)self);
+    if (!iter) {
+        return NULL;
+    }
+    if (!tchdbiterinit(self->hdb)) {
+        Py_DECREF(iter);
+        return set_hdb_error(self->hdb, NULL);
+    }
+    self->changed = false;
+    return iter;
+}
+
+
+/* HDBIterKeysType.tp_iternext */
+static PyObject *
+HDBIterKeys_tp_iternext(DBIter *self)
+{
+    HDB *hdb = (HDB *)self->db;
+    void *key;
+    int key_size;
+    PyObject *pykey;
+
+    if (hdb->changed) {
+        return set_error(Error, "HDB changed during iteration");
+    }
+    key = tchdbiternext(hdb->hdb, &key_size);
+    if (!key) {
+        if (tchdbecode(hdb->hdb) == TCENOREC) {
+            return set_stopiteration_error();
+        }
+        return set_hdb_error(hdb->hdb, NULL);
+    }
+    pykey = void_to_bytes(key, key_size);
+    tcfree(key);
+    return pykey;
+}
+
+
+/* HDBIterKeysType */
+static PyTypeObject HDBIterKeysType = {
+    PyVarObject_HEAD_INIT(NULL, 0)
+    "tokyo.cabinet.HDBIterKeys",              /*tp_name*/
+    sizeof(DBIter),                           /*tp_basicsize*/
+    0,                                        /*tp_itemsize*/
+    (destructor)DBIter_tp_dealloc,            /*tp_dealloc*/
+    0,                                        /*tp_print*/
+    0,                                        /*tp_getattr*/
+    0,                                        /*tp_setattr*/
+    0,                                        /*tp_compare*/
+    0,                                        /*tp_repr*/
+    0,                                        /*tp_as_number*/
+    0,                                        /*tp_as_sequence*/
+    0,                                        /*tp_as_mapping*/
+    0,                                        /*tp_hash */
+    0,                                        /*tp_call*/
+    0,                                        /*tp_str*/
+    0,                                        /*tp_getattro*/
+    0,                                        /*tp_setattro*/
+    0,                                        /*tp_as_buffer*/
+    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_HAVE_GC,  /*tp_flags*/
+    0,                                        /*tp_doc*/
+    (traverseproc)DBIter_tp_traverse,         /*tp_traverse*/
+    (inquiry)DBIter_tp_clear,                 /*tp_clear*/
+    0,                                        /*tp_richcompare*/
+    0,                                        /*tp_weaklistoffset*/
+    PyObject_SelfIter,                        /*tp_iter*/
+    (iternextfunc)HDBIterKeys_tp_iternext,    /*tp_iternext*/
+    DBIter_tp_methods,                        /*tp_methods*/
+};
+
+
+/* HDBIterValuesType.tp_iternext */
+static PyObject *
+HDBIterValues_tp_iternext(DBIter *self)
+{
+    HDB *hdb = (HDB *)self->db;
+    TCXSTR *key, *value;
+    PyObject *pyvalue = NULL;
+
+    if (hdb->changed) {
+        return set_error(Error, "HDB changed during iteration");
+    }
+    key = tcxstrnew();
+    value = tcxstrnew();
+    if (!tchdbiternext3(hdb->hdb, key, value)) {
+        if (tchdbecode(hdb->hdb) == TCENOREC) {
+            set_stopiteration_error();
+        }
+        else {
+            set_hdb_error(hdb->hdb, NULL);
+        }
+    }
+    else {
+        pyvalue = tcxstr_to_bytes(value);
+    }
+    tcxstrdel(key);
+    tcxstrdel(value);
+    return pyvalue;
+}
+
+
+/* HDBIterValuesType */
+static PyTypeObject HDBIterValuesType = {
+    PyVarObject_HEAD_INIT(NULL, 0)
+    "tokyo.cabinet.HDBIterValues",            /*tp_name*/
+    sizeof(DBIter),                           /*tp_basicsize*/
+    0,                                        /*tp_itemsize*/
+    (destructor)DBIter_tp_dealloc,            /*tp_dealloc*/
+    0,                                        /*tp_print*/
+    0,                                        /*tp_getattr*/
+    0,                                        /*tp_setattr*/
+    0,                                        /*tp_compare*/
+    0,                                        /*tp_repr*/
+    0,                                        /*tp_as_number*/
+    0,                                        /*tp_as_sequence*/
+    0,                                        /*tp_as_mapping*/
+    0,                                        /*tp_hash */
+    0,                                        /*tp_call*/
+    0,                                        /*tp_str*/
+    0,                                        /*tp_getattro*/
+    0,                                        /*tp_setattro*/
+    0,                                        /*tp_as_buffer*/
+    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_HAVE_GC,  /*tp_flags*/
+    0,                                        /*tp_doc*/
+    (traverseproc)DBIter_tp_traverse,         /*tp_traverse*/
+    (inquiry)DBIter_tp_clear,                 /*tp_clear*/
+    0,                                        /*tp_richcompare*/
+    0,                                        /*tp_weaklistoffset*/
+    PyObject_SelfIter,                        /*tp_iter*/
+    (iternextfunc)HDBIterValues_tp_iternext,  /*tp_iternext*/
+    DBIter_tp_methods,                        /*tp_methods*/
+};
+
+
+/* HDBIterItemsType.tp_iternext */
+static PyObject *
+HDBIterItems_tp_iternext(DBIter *self)
+{
+    HDB *hdb = (HDB *)self->db;
+    TCXSTR *key, *value;
+    PyObject *pykey, *pyvalue, *pyresult = NULL;
+
+    if (hdb->changed) {
+        return set_error(Error, "HDB changed during iteration");
+    }
+    key = tcxstrnew();
+    value = tcxstrnew();
+    if (!tchdbiternext3(hdb->hdb, key, value)) {
+        if (tchdbecode(hdb->hdb) == TCENOREC) {
+            set_stopiteration_error();
+        }
+        else {
+            set_hdb_error(hdb->hdb, NULL);
+        }
+    }
+    else {
+        pykey = tcxstr_to_bytes(key);
+        pyvalue = tcxstr_to_bytes(value);
+        if (pykey && pyvalue) {
+            pyresult = PyTuple_Pack(2, pykey, pyvalue);
+        }
+        Py_XDECREF(pykey);
+        Py_XDECREF(pyvalue);
+    }
+    tcxstrdel(key);
+    tcxstrdel(value);
+    return pyresult;
+}
+
+
+/* HDBIterItemsType */
+static PyTypeObject HDBIterItemsType = {
+    PyVarObject_HEAD_INIT(NULL, 0)
+    "tokyo.cabinet.HDBIterItems",             /*tp_name*/
+    sizeof(DBIter),                           /*tp_basicsize*/
+    0,                                        /*tp_itemsize*/
+    (destructor)DBIter_tp_dealloc,            /*tp_dealloc*/
+    0,                                        /*tp_print*/
+    0,                                        /*tp_getattr*/
+    0,                                        /*tp_setattr*/
+    0,                                        /*tp_compare*/
+    0,                                        /*tp_repr*/
+    0,                                        /*tp_as_number*/
+    0,                                        /*tp_as_sequence*/
+    0,                                        /*tp_as_mapping*/
+    0,                                        /*tp_hash */
+    0,                                        /*tp_call*/
+    0,                                        /*tp_str*/
+    0,                                        /*tp_getattro*/
+    0,                                        /*tp_setattro*/
+    0,                                        /*tp_as_buffer*/
+    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_HAVE_GC,  /*tp_flags*/
+    0,                                        /*tp_doc*/
+    (traverseproc)DBIter_tp_traverse,         /*tp_traverse*/
+    (inquiry)DBIter_tp_clear,                 /*tp_clear*/
+    0,                                        /*tp_richcompare*/
+    0,                                        /*tp_weaklistoffset*/
+    PyObject_SelfIter,                        /*tp_iter*/
+    (iternextfunc)HDBIterItems_tp_iternext,   /*tp_iternext*/
+    DBIter_tp_methods,                        /*tp_methods*/
+};
+
+
 /*******************************************************************************
 * HDBType
 *******************************************************************************/
@@ -69,13 +280,13 @@ HDB_tp_new(PyTypeObject *type, PyObject *args, PyObject *kwargs)
 static int
 HDB_Contains(HDB *self, PyObject *pykey)
 {
-    const char *key, *value;
+    void *key, *value;
+    int key_size, value_size;
 
-    key = PyBytes_AsString(pykey);
-    if (!key) {
+    if (bytes_to_void(pykey, &key, &key_size)) {
         return -1;
     }
-    value = tchdbget2(self->hdb, key);
+    value = tchdbget(self->hdb, key, key_size, &value_size);
     if (!value) {
         if (tchdbecode(self->hdb) == TCENOREC) {
             return 0;
@@ -83,7 +294,7 @@ HDB_Contains(HDB *self, PyObject *pykey)
         set_hdb_error(self->hdb, NULL);
         return -1;
     }
-    tcfree((void *)value);
+    tcfree(value);
     return 1;
 }
 
@@ -105,7 +316,7 @@ static PySequenceMethods HDB_tp_as_sequence = {
 static Py_ssize_t
 HDB_Length(HDB *self)
 {
-    return (Py_ssize_t)tchdbrnum(self->hdb);
+    return DB_Length(tchdbrnum(self->hdb));
 }
 
 
@@ -113,19 +324,19 @@ HDB_Length(HDB *self)
 static PyObject *
 HDB_GetItem(HDB *self, PyObject *pykey)
 {
-    const char *key, *value;
+    void *key, *value;
+    int key_size, value_size;
     PyObject *pyvalue;
 
-    key = PyBytes_AsString(pykey);
-    if (!key) {
+    if (bytes_to_void(pykey, &key, &key_size)) {
         return NULL;
     }
-    value = tchdbget2(self->hdb, key);
+    value = tchdbget(self->hdb, key, key_size, &value_size);
     if (!value) {
         return set_hdb_error(self->hdb, key);
     }
-    pyvalue = PyBytes_FromString(value);
-    tcfree((void *)value);
+    pyvalue = void_to_bytes(value, value_size);
+    tcfree(value);
     return pyvalue;
 }
 
@@ -134,24 +345,23 @@ HDB_GetItem(HDB *self, PyObject *pykey)
 static int
 HDB_SetItem(HDB *self, PyObject *pykey, PyObject *pyvalue)
 {
-    const char *key, *value;
+    void *key, *value;
+    int key_size, value_size;
 
-    key = PyBytes_AsString(pykey);
-    if (!key) {
+    if (bytes_to_void(pykey, &key, &key_size)) {
         return -1;
     }
     if (pyvalue) {
-        value = PyBytes_AsString(pyvalue);
-        if (!value) {
+        if (bytes_to_void(pyvalue, &value, &value_size)) {
             return -1;
         }
-        if (!tchdbput2(self->hdb, key, value)) {
+        if (!tchdbput(self->hdb, key, key_size, value, value_size)) {
             set_hdb_error(self->hdb, NULL);
             return -1;
         }
     }
     else {
-        if (!tchdbout2(self->hdb, key)) {
+        if (!tchdbout(self->hdb, key, key_size)) {
             set_hdb_error(self->hdb, key);
             return -1;
         }
@@ -173,46 +383,7 @@ static PyMappingMethods HDB_tp_as_mapping = {
 static PyObject *
 HDB_tp_iter(HDB *self)
 {
-    if (!tchdbiterinit(self->hdb)) {
-        return set_hdb_error(self->hdb, NULL);
-    }
-    self->changed = false;
-    Py_INCREF(self);
-    return (PyObject *)self;
-}
-
-
-/* HDBType.tp_iternext */
-static PyObject *
-HDB_tp_iternext(HDB *self)
-{
-    const char *key;
-    PyObject *pykey;
-
-    if (self->changed) {
-        return set_error(Error, "HDB changed during iteration");
-    }
-    key = tchdbiternext2(self->hdb);
-    if (!key) {
-        if (tchdbecode(self->hdb) == TCENOREC) {
-            return set_stopiteration_error();
-        }
-        return set_hdb_error(self->hdb, NULL);
-    }
-    pykey = PyBytes_FromString(key);
-    tcfree((void *)key);
-    return pykey;
-}
-
-
-/* HDB.__length_hint__ */
-PyDoc_STRVAR(HDB_length_hint_doc,
-"Private method returning an estimate of len(list(hdb)).");
-
-static PyObject *
-HDB_length_hint(HDB *self)
-{
-    return PyLong_FromSsize_t(HDB_Length(self));
+    return new_HDBIter(self, &HDBIterKeysType);
 }
 
 
@@ -421,18 +592,18 @@ put), this method raises KeyError if key is already in the database.");
 static PyObject *
 HDB_putkeep(HDB *self, PyObject *args)
 {
-    const char *key, *value;
+    void *key, *value;
+    int key_size, value_size;
     PyObject *pykey, *pyvalue;
 
     if (!PyArg_ParseTuple(args, "OO:putkeep", &pykey, &pyvalue)) {
         return NULL;
     }
-    key = PyBytes_AsString(pykey);
-    value = PyBytes_AsString(pyvalue);
-    if (!(key && value)) {
+    if (bytes_to_void(pykey, &key, &key_size) ||
+        bytes_to_void(pyvalue, &value, &value_size)) {
         return NULL;
     }
-    if (!tchdbputkeep2(self->hdb, key, value)) {
+    if (!tchdbputkeep(self->hdb, key, key_size, value, value_size)) {
         return set_hdb_error(self->hdb, key);
     }
     self->changed = true;
@@ -450,18 +621,18 @@ If there is no corresponding record, a new record is stored.");
 static PyObject *
 HDB_putcat(HDB *self, PyObject *args)
 {
-    const char *key, *value;
+    void *key, *value;
+    int key_size, value_size;
     PyObject *pykey, *pyvalue;
 
     if (!PyArg_ParseTuple(args, "OO:putcat", &pykey, &pyvalue)) {
         return NULL;
     }
-    key = PyBytes_AsString(pykey);
-    value = PyBytes_AsString(pyvalue);
-    if (!(key && value)) {
+    if (bytes_to_void(pykey, &key, &key_size) ||
+        bytes_to_void(pyvalue, &value, &value_size)) {
         return NULL;
     }
-    if (!tchdbputcat2(self->hdb, key, value)) {
+    if (!tchdbputcat(self->hdb, key, key_size, value, value_size)) {
         return set_hdb_error(self->hdb, NULL);
     }
     self->changed = true;
@@ -480,18 +651,18 @@ into the file at a ?blast? (when?, relation to sync()?).");
 static PyObject *
 HDB_putasync(HDB *self, PyObject *args)
 {
-    const char *key, *value;
+    void *key, *value;
+    int key_size, value_size;
     PyObject *pykey, *pyvalue;
 
     if (!PyArg_ParseTuple(args, "OO:putasync", &pykey, &pyvalue)) {
         return NULL;
     }
-    key = PyBytes_AsString(pykey);
-    value = PyBytes_AsString(pyvalue);
-    if (!(key && value)) {
+    if (bytes_to_void(pykey, &key, &key_size) ||
+        bytes_to_void(pyvalue, &value, &value_size)) {
         return NULL;
     }
-    if (!tchdbputasync2(self->hdb, key, value)) {
+    if (!tchdbputasync(self->hdb, key, key_size, value, value_size)) {
         return set_hdb_error(self->hdb, NULL);
     }
     self->changed = true;
@@ -526,20 +697,19 @@ is applied.");
 static PyObject *
 HDB_searchkeys(HDB *self, PyObject *args)
 {
-    const char *prefix;
-    int max = -1;
+    void *prefix;
+    int prefix_size, max = -1;
     TCLIST *result;
     PyObject *pyprefix, *pyresult;
 
     if (!PyArg_ParseTuple(args, "O|i:searchkeys", &pyprefix, &max)) {
         return NULL;
     }
-    prefix = PyBytes_AsString(pyprefix);
-    if (!prefix) {
+    if (bytes_to_void(pyprefix, &prefix, &prefix_size)) {
         return NULL;
     }
     Py_BEGIN_ALLOW_THREADS
-    result = tchdbfwmkeys2(self->hdb, prefix, max);
+    result = tchdbfwmkeys(self->hdb, prefix, prefix_size, max);
     Py_END_ALLOW_THREADS
     pyresult = tclist_to_frozenset(result);
     tclistdel(result);
@@ -721,22 +891,23 @@ static PyObject *
 HDB_addint(HDB *self, PyObject *args)
 {
     PyObject *pykey;
-    char *key;
-    Py_ssize_t key_size;
-    int num, result;
+    void *key;
+    int key_size, num, result;
 
     if (!PyArg_ParseTuple(args, "Oi:addint", &pykey, &num)) {
         return NULL;
     }
-    if(PyBytes_AsStringAndSize(pykey, &key, &key_size)) {
+    if (bytes_to_void(pykey, &key, &key_size)) {
         return NULL;
     }
-    result = tchdbaddint(self->hdb, key, (int)key_size, num);
+    result = tchdbaddint(self->hdb, key, key_size, num);
     if (result == INT_MIN && tchdbecode(self->hdb) != TCESUCCESS) {
         return set_hdb_error(self->hdb, key);
     }
-    self->changed = true;
-    return Py_BuildValue("i", result);
+    if (num) {
+        self->changed = true;
+    }
+    return PyInt_FromLong((long)result);
 }
 
 
@@ -757,29 +928,68 @@ static PyObject *
 HDB_adddouble(HDB *self, PyObject *args)
 {
     PyObject *pykey;
-    char *key;
-    Py_ssize_t key_size;
+    void *key;
+    int key_size;
     double num, result;
 
     if (!PyArg_ParseTuple(args, "Od:adddouble", &pykey, &num)) {
         return NULL;
     }
-    if(PyBytes_AsStringAndSize(pykey, &key, &key_size)) {
+    if (bytes_to_void(pykey, &key, &key_size)) {
         return NULL;
     }
-    result = tchdbadddouble(self->hdb, key, (int)key_size, num);
+    result = tchdbadddouble(self->hdb, key, key_size, num);
     if (Py_IS_NAN(result)) {
         return set_hdb_error(self->hdb, key);
     }
-    self->changed = true;
+    if (num) {
+        self->changed = true;
+    }
     return PyFloat_FromDouble(result);
 }
 
 
+/* HDB.iterkeys() */
+PyDoc_STRVAR(HDB_iterkeys_doc,
+"iterkeys()\n\
+\n\
+Return an iterator over the database's keys.");
+
+static PyObject *
+HDB_iterkeys(HDB *self)
+{
+    return new_HDBIter(self, &HDBIterKeysType);
+}
+
+
+/* HDB.itervalues() */
+PyDoc_STRVAR(HDB_itervalues_doc,
+"itervalues()\n\
+\n\
+Return an iterator over the database's values.");
+
+static PyObject *
+HDB_itervalues(HDB *self)
+{
+    return new_HDBIter(self, &HDBIterValuesType);
+}
+
+
+/* HDB.iteritems() */
+PyDoc_STRVAR(HDB_iteritems_doc,
+"iteritems()\n\
+\n\
+Return an iterator over the database's items.");
+
+static PyObject *
+HDB_iteritems(HDB *self)
+{
+    return new_HDBIter(self, &HDBIterItemsType);
+}
+
+
 /* HDBType.tp_methods */
 static PyMethodDef HDB_tp_methods[] = {
-    {"__length_hint__", (PyCFunction)HDB_length_hint, METH_NOARGS,
-     HDB_length_hint_doc},
     {"open", (PyCFunction)HDB_open, METH_VARARGS, HDB_open_doc},
     {"close", (PyCFunction)HDB_close, METH_NOARGS, HDB_close_doc},
     {"clear", (PyCFunction)HDB_clear, METH_NOARGS, HDB_clear_doc},
@@ -804,6 +1014,9 @@ static PyMethodDef HDB_tp_methods[] = {
     {"setdfunit", (PyCFunction)HDB_setdfunit, METH_VARARGS, HDB_setdfunit_doc},
     {"addint", (PyCFunction)HDB_addint, METH_VARARGS, HDB_addint_doc},
     {"adddouble", (PyCFunction)HDB_adddouble, METH_VARARGS, HDB_adddouble_doc},
+    {"iterkeys", (PyCFunction)HDB_iterkeys, METH_NOARGS, HDB_iterkeys_doc},
+    {"itervalues", (PyCFunction)HDB_itervalues, METH_NOARGS, HDB_itervalues_doc},
+    {"iteritems", (PyCFunction)HDB_iteritems, METH_NOARGS, HDB_iteritems_doc},
     {NULL}  /* Sentinel */
 };
 
@@ -819,7 +1032,7 @@ HDB_path_get(HDB *self, void *closure)
 
     path = tchdbpath(self->hdb);
     if (path) {
-        return Py_BuildValue("s", path);
+        return PyString_FromString(path);
     }
     Py_RETURN_NONE;
 }
@@ -872,7 +1085,7 @@ static PyTypeObject HDBType = {
     0,                                        /*tp_richcompare*/
     0,                                        /*tp_weaklistoffset*/
     (getiterfunc)HDB_tp_iter,                 /*tp_iter*/
-    (iternextfunc)HDB_tp_iternext,            /*tp_iternext*/
+    0,                                        /*tp_iternext*/
     HDB_tp_methods,                           /*tp_methods*/
     0,                                        /*tp_members*/
     HDB_tp_getsets,                           /*tp_getsets*/
