@@ -171,6 +171,22 @@ RTDBQuery_filter(RTDBQuery *self, PyObject *args)
 }
 
 
+/* RTDBQuery.count() */
+PyDoc_STRVAR(RTDBQuery_count_doc,
+"Returns the length of the result set.");
+
+static PyObject *
+RTDBQuery_count(RTDBQuery *self)
+{
+    int count;
+
+    Py_BEGIN_ALLOW_THREADS
+    count = tcrdbqrysearchcount(self->rqry);
+    Py_END_ALLOW_THREADS
+    return PyInt_FromLong((long)count);
+}
+
+
 /* RTDBQueryType.tp_methods */
 static PyMethodDef RTDBQuery_tp_methods[] = {
     {"search", (PyCFunction)RTDBQuery_search, METH_NOARGS, RTDBQuery_search_doc},
@@ -178,21 +194,11 @@ static PyMethodDef RTDBQuery_tp_methods[] = {
     {"sort", (PyCFunction)RTDBQuery_sort, METH_VARARGS, RTDBQuery_sort_doc},
     {"limit", (PyCFunction)RTDBQuery_limit, METH_VARARGS, RTDBQuery_limit_doc},
     {"filter", (PyCFunction)RTDBQuery_filter, METH_VARARGS, RTDBQuery_filter_doc},
+    {"count", (PyCFunction)RTDBQuery_count, METH_NOARGS, RTDBQuery_count_doc},
     {NULL}  /* Sentinel */
 };
 
 
-/* RTDBQuery.count */
-PyDoc_STRVAR(RTDBQuery_count_doc,
-"TODO.");
-
-static PyObject *
-RTDBQuery_count_get(RTDBQuery *self, void *closure)
-{
-    return Py_BuildValue("i", tcrdbqrysearchcount(self->rqry));
-}
-
-
 /* RTDBQuery.hint */
 PyDoc_STRVAR(RTDBQuery_hint_doc,
 "TODO.");
@@ -200,13 +206,12 @@ PyDoc_STRVAR(RTDBQuery_hint_doc,
 static PyObject *
 RTDBQuery_hint_get(RTDBQuery *self, void *closure)
 {
-    return Py_BuildValue("s", tcrdbqryhint(self->rqry));
+    return PyString_FromString(tcrdbqryhint(self->rqry));
 }
 
 
 /* RTDBQueryType.tp_getsets */
 static PyGetSetDef RTDBQuery_tp_getsets[] = {
-    {"count", (getter)RTDBQuery_count_get, NULL, RTDBQuery_count_doc, NULL},
     {"hint", (getter)RTDBQuery_hint_get, NULL, RTDBQuery_hint_doc, NULL},
     {NULL}  /* Sentinel */
 };
@@ -247,6 +252,281 @@ static PyTypeObject RTDBQueryType = {
 };
 
 
+/*******************************************************************************
+* RTDB iterator types
+*******************************************************************************/
+
+/* RTDBIterValuesType.tp_iternext */
+static PyObject *
+RTDBIterValues_tp_iternext(DBIter *self)
+{
+    RDBBase *rdbbase = (RDBBase *)self->db;
+    void *key;
+    int key_size;
+    TCMAP *value;
+    PyObject *pyvalue;
+
+    if (rdbbase->changed) {
+        return set_error(Error, "DB changed during iteration");
+    }
+    Py_BEGIN_ALLOW_THREADS
+    key = tcrdbiternext(rdbbase->rdb, &key_size);
+    if (key) {
+        value = tcrdbtblget(rdbbase->rdb, key, key_size);
+    }
+    Py_END_ALLOW_THREADS
+    if (!key) {
+        if (tcrdbecode(rdbbase->rdb) == TTENOREC) {
+            return set_stopiteration_error();
+        }
+        return set_rdb_error(rdbbase->rdb, NULL);
+    }
+    pyvalue = tcmap_to_dict(value);
+    tcfree(key);
+    tcmapdel(value);
+    return pyvalue;
+}
+
+
+/* RTDBIterValuesType */
+static PyTypeObject RTDBIterValuesType = {
+    PyVarObject_HEAD_INIT(NULL, 0)
+    "tokyo.cabinet.RTDBIterValues",            /*tp_name*/
+    sizeof(DBIter),                           /*tp_basicsize*/
+    0,                                        /*tp_itemsize*/
+    (destructor)DBIter_tp_dealloc,            /*tp_dealloc*/
+    0,                                        /*tp_print*/
+    0,                                        /*tp_getattr*/
+    0,                                        /*tp_setattr*/
+    0,                                        /*tp_compare*/
+    0,                                        /*tp_repr*/
+    0,                                        /*tp_as_number*/
+    0,                                        /*tp_as_sequence*/
+    0,                                        /*tp_as_mapping*/
+    0,                                        /*tp_hash */
+    0,                                        /*tp_call*/
+    0,                                        /*tp_str*/
+    0,                                        /*tp_getattro*/
+    0,                                        /*tp_setattro*/
+    0,                                        /*tp_as_buffer*/
+    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_HAVE_GC,  /*tp_flags*/
+    0,                                        /*tp_doc*/
+    (traverseproc)DBIter_tp_traverse,         /*tp_traverse*/
+    (inquiry)DBIter_tp_clear,                 /*tp_clear*/
+    0,                                        /*tp_richcompare*/
+    0,                                        /*tp_weaklistoffset*/
+    PyObject_SelfIter,                        /*tp_iter*/
+    (iternextfunc)RTDBIterValues_tp_iternext,  /*tp_iternext*/
+    DBIter_tp_methods,                        /*tp_methods*/
+};
+
+
+/* RTDBIterItemsType.tp_iternext */
+static PyObject *
+RTDBIterItems_tp_iternext(DBIter *self)
+{
+    RDBBase *rdbbase = (RDBBase *)self->db;
+    void *key;
+    int key_size;
+    TCMAP *value;
+    PyObject *pykey, *pyvalue, *pyresult = NULL;
+
+    if (rdbbase->changed) {
+        return set_error(Error, "DB changed during iteration");
+    }
+    Py_BEGIN_ALLOW_THREADS
+    key = tcrdbiternext(rdbbase->rdb, &key_size);
+    if (key) {
+        value = tcrdbtblget(rdbbase->rdb, key, key_size);
+    }
+    Py_END_ALLOW_THREADS
+    if (!key) {
+        if (tcrdbecode(rdbbase->rdb) == TTENOREC) {
+            return set_stopiteration_error();
+        }
+        return set_rdb_error(rdbbase->rdb, NULL);
+    }
+    pykey = void_to_bytes(key, key_size);
+    pyvalue = tcmap_to_dict(value);
+    if (pykey && pyvalue) {
+        pyresult = PyTuple_Pack(2, pykey, pyvalue);
+    }
+    Py_XDECREF(pykey);
+    Py_XDECREF(pyvalue);
+    tcfree(key);
+    tcmapdel(value);
+    return pyresult;
+}
+
+
+/* RTDBIterItemsType */
+static PyTypeObject RTDBIterItemsType = {
+    PyVarObject_HEAD_INIT(NULL, 0)
+    "tokyo.cabinet.RTDBIterItems",             /*tp_name*/
+    sizeof(DBIter),                           /*tp_basicsize*/
+    0,                                        /*tp_itemsize*/
+    (destructor)DBIter_tp_dealloc,            /*tp_dealloc*/
+    0,                                        /*tp_print*/
+    0,                                        /*tp_getattr*/
+    0,                                        /*tp_setattr*/
+    0,                                        /*tp_compare*/
+    0,                                        /*tp_repr*/
+    0,                                        /*tp_as_number*/
+    0,                                        /*tp_as_sequence*/
+    0,                                        /*tp_as_mapping*/
+    0,                                        /*tp_hash */
+    0,                                        /*tp_call*/
+    0,                                        /*tp_str*/
+    0,                                        /*tp_getattro*/
+    0,                                        /*tp_setattro*/
+    0,                                        /*tp_as_buffer*/
+    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_HAVE_GC,  /*tp_flags*/
+    0,                                        /*tp_doc*/
+    (traverseproc)DBIter_tp_traverse,         /*tp_traverse*/
+    (inquiry)DBIter_tp_clear,                 /*tp_clear*/
+    0,                                        /*tp_richcompare*/
+    0,                                        /*tp_weaklistoffset*/
+    PyObject_SelfIter,                        /*tp_iter*/
+    (iternextfunc)RTDBIterItems_tp_iternext,   /*tp_iternext*/
+    DBIter_tp_methods,                        /*tp_methods*/
+};
+
+
+/* RTDBIterValuesKeysType.tp_iternext */
+static PyObject *
+RTDBIterValuesKeys_tp_iternext(DBIter *self)
+{
+    RDBBase *rdbbase = (RDBBase *)self->db;
+    void *key;
+    int key_size;
+    TCMAP *value;
+    TCLIST *valuekeys;
+    PyObject *pyvaluekeys;
+
+    if (rdbbase->changed) {
+        return set_error(Error, "DB changed during iteration");
+    }
+    Py_BEGIN_ALLOW_THREADS
+    key = tcrdbiternext(rdbbase->rdb, &key_size);
+    if (key) {
+        value = tcrdbtblget(rdbbase->rdb, key, key_size);
+    }
+    Py_END_ALLOW_THREADS
+    if (!key) {
+        if (tcrdbecode(rdbbase->rdb) == TTENOREC) {
+            return set_stopiteration_error();
+        }
+        return set_rdb_error(rdbbase->rdb, NULL);
+    }
+    valuekeys = tcmapkeys(value);
+    pyvaluekeys = tclist_to_tuple(valuekeys);
+    tcfree(key);
+    tcmapdel(value);
+    tclistdel(valuekeys);
+    return pyvaluekeys;
+}
+
+
+/* RTDBIterValuesKeysType */
+static PyTypeObject RTDBIterValuesKeysType = {
+    PyVarObject_HEAD_INIT(NULL, 0)
+    "tokyo.cabinet.RTDBIterValuesKeys",       /*tp_name*/
+    sizeof(DBIter),                           /*tp_basicsize*/
+    0,                                        /*tp_itemsize*/
+    (destructor)DBIter_tp_dealloc,            /*tp_dealloc*/
+    0,                                        /*tp_print*/
+    0,                                        /*tp_getattr*/
+    0,                                        /*tp_setattr*/
+    0,                                        /*tp_compare*/
+    0,                                        /*tp_repr*/
+    0,                                        /*tp_as_number*/
+    0,                                        /*tp_as_sequence*/
+    0,                                        /*tp_as_mapping*/
+    0,                                        /*tp_hash */
+    0,                                        /*tp_call*/
+    0,                                        /*tp_str*/
+    0,                                        /*tp_getattro*/
+    0,                                        /*tp_setattro*/
+    0,                                        /*tp_as_buffer*/
+    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_HAVE_GC,  /*tp_flags*/
+    0,                                        /*tp_doc*/
+    (traverseproc)DBIter_tp_traverse,         /*tp_traverse*/
+    (inquiry)DBIter_tp_clear,                 /*tp_clear*/
+    0,                                        /*tp_richcompare*/
+    0,                                        /*tp_weaklistoffset*/
+    PyObject_SelfIter,                        /*tp_iter*/
+    (iternextfunc)RTDBIterValuesKeys_tp_iternext, /*tp_iternext*/
+    DBIter_tp_methods,                        /*tp_methods*/
+};
+
+
+/* RTDBIterValuesValsType.tp_iternext */
+static PyObject *
+RTDBIterValuesVals_tp_iternext(DBIter *self)
+{
+    RDBBase *rdbbase = (RDBBase *)self->db;
+    void *key;
+    int key_size;
+    TCMAP *value;
+    TCLIST *valuevals;
+    PyObject *pyvaluevals;
+
+    if (rdbbase->changed) {
+        return set_error(Error, "DB changed during iteration");
+    }
+    Py_BEGIN_ALLOW_THREADS
+    key = tcrdbiternext(rdbbase->rdb, &key_size);
+    if (key) {
+        value = tcrdbtblget(rdbbase->rdb, key, key_size);
+    }
+    Py_END_ALLOW_THREADS
+    if (!key) {
+        if (tcrdbecode(rdbbase->rdb) == TTENOREC) {
+            return set_stopiteration_error();
+        }
+        return set_rdb_error(rdbbase->rdb, NULL);
+    }
+    valuevals = tcmapvals(value);
+    pyvaluevals = tclist_to_tuple(valuevals);
+    tcmapdel(value);
+    tclistdel(valuevals);
+    return pyvaluevals;
+}
+
+
+/* RTDBIterValuesValsType */
+static PyTypeObject RTDBIterValuesValsType = {
+    PyVarObject_HEAD_INIT(NULL, 0)
+    "tokyo.cabinet.RTDBIterValuesVals",       /*tp_name*/
+    sizeof(DBIter),                           /*tp_basicsize*/
+    0,                                        /*tp_itemsize*/
+    (destructor)DBIter_tp_dealloc,            /*tp_dealloc*/
+    0,                                        /*tp_print*/
+    0,                                        /*tp_getattr*/
+    0,                                        /*tp_setattr*/
+    0,                                        /*tp_compare*/
+    0,                                        /*tp_repr*/
+    0,                                        /*tp_as_number*/
+    0,                                        /*tp_as_sequence*/
+    0,                                        /*tp_as_mapping*/
+    0,                                        /*tp_hash */
+    0,                                        /*tp_call*/
+    0,                                        /*tp_str*/
+    0,                                        /*tp_getattro*/
+    0,                                        /*tp_setattro*/
+    0,                                        /*tp_as_buffer*/
+    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_HAVE_GC,  /*tp_flags*/
+    0,                                        /*tp_doc*/
+    (traverseproc)DBIter_tp_traverse,         /*tp_traverse*/
+    (inquiry)DBIter_tp_clear,                 /*tp_clear*/
+    0,                                        /*tp_richcompare*/
+    0,                                        /*tp_weaklistoffset*/
+    PyObject_SelfIter,                        /*tp_iter*/
+    (iternextfunc)RTDBIterValuesVals_tp_iternext, /*tp_iternext*/
+    DBIter_tp_methods,                        /*tp_methods*/
+};
+
+
 /*******************************************************************************
 * RTDBType
 *******************************************************************************/
@@ -266,16 +546,16 @@ http://1978th.net/tokyotyrant/spex.html#tcrdbapi_apitbl");
 static
 int RTDB_Contains(RTDB *self, PyObject *pykey)
 {
-    char *key;
-    Py_ssize_t key_size;
+    void *key;
+    int key_size;
     TCMAP *value;
     RDBBase *rdbbase = (RDBBase *)self;
 
-    if(PyBytes_AsStringAndSize(pykey, &key, &key_size)) {
+    if (bytes_to_void(pykey, &key, &key_size)) {
         return -1;
     }
     Py_BEGIN_ALLOW_THREADS
-    value = tcrdbtblget(rdbbase->rdb, key, (int)key_size);
+    value = tcrdbtblget(rdbbase->rdb, key, key_size);
     Py_END_ALLOW_THREADS
     if (!value) {
         if (tcrdbecode(rdbbase->rdb) == TTENOREC) {
@@ -306,17 +586,17 @@ static PySequenceMethods RTDB_tp_as_sequence = {
 static PyObject *
 RTDB_GetItem(RTDB *self, PyObject *pykey)
 {
-    char *key;
-    Py_ssize_t key_size;
+    void *key;
+    int key_size;
     TCMAP *value;
     PyObject *pyvalue;
     RDBBase *rdbbase = (RDBBase *)self;
 
-    if(PyBytes_AsStringAndSize(pykey, &key, &key_size)) {
+    if (bytes_to_void(pykey, &key, &key_size)) {
         return NULL;
     }
     Py_BEGIN_ALLOW_THREADS
-    value = tcrdbtblget(rdbbase->rdb, key, (int)key_size);
+    value = tcrdbtblget(rdbbase->rdb, key, key_size);
     Py_END_ALLOW_THREADS
     if (!value) {
         return set_rdb_error(rdbbase->rdb, key);
@@ -331,13 +611,13 @@ RTDB_GetItem(RTDB *self, PyObject *pykey)
 static int
 RTDB_SetItem(RTDB *self, PyObject *pykey, PyObject *pyvalue)
 {
-    char *key;
-    Py_ssize_t key_size;
+    void *key;
+    int key_size;
     TCMAP *value;
     bool result;
     RDBBase *rdbbase = (RDBBase *)self;
 
-    if(PyBytes_AsStringAndSize(pykey, &key, &key_size)) {
+    if (bytes_to_void(pykey, &key, &key_size)) {
         return -1;
     }
     if (pyvalue) {
@@ -346,7 +626,7 @@ RTDB_SetItem(RTDB *self, PyObject *pykey, PyObject *pyvalue)
             return -1;
         }
         Py_BEGIN_ALLOW_THREADS
-        result = tcrdbtblput(rdbbase->rdb, key, (int)key_size, value);
+        result = tcrdbtblput(rdbbase->rdb, key, key_size, value);
         Py_END_ALLOW_THREADS
         if (!result) {
             tcmapdel(value);
@@ -357,7 +637,7 @@ RTDB_SetItem(RTDB *self, PyObject *pykey, PyObject *pyvalue)
     }
     else {
         Py_BEGIN_ALLOW_THREADS
-        result = tcrdbtblout(rdbbase->rdb, key, (int)key_size);
+        result = tcrdbtblout(rdbbase->rdb, key, key_size);
         Py_END_ALLOW_THREADS
         if (!result) {
             set_rdb_error(rdbbase->rdb, key);
@@ -476,8 +756,8 @@ put), this method raises KeyError if key is already in the database.");
 static PyObject *
 RTDB_putkeep(RTDB *self, PyObject *args, PyObject *kwargs)
 {
-    char *key;
-    Py_ssize_t key_size;
+    void *key;
+    int key_size;
     TCMAP *value;
     PyObject *pykey, *pyvalue = NULL;
     bool result;
@@ -491,7 +771,7 @@ RTDB_putkeep(RTDB *self, PyObject *args, PyObject *kwargs)
     if (!pyvalue) {
         return NULL;
     }
-    if(PyBytes_AsStringAndSize(pykey, &key, &key_size)) {
+    if (bytes_to_void(pykey, &key, &key_size)) {
         return NULL;
     }
     value = dict_to_tcmap(pyvalue);
@@ -499,7 +779,7 @@ RTDB_putkeep(RTDB *self, PyObject *args, PyObject *kwargs)
         return NULL;
     }
     Py_BEGIN_ALLOW_THREADS
-    result = tcrdbtblputkeep(rdbbase->rdb, key, (int)key_size, value);
+    result = tcrdbtblputkeep(rdbbase->rdb, key, key_size, value);
     tcmapdel(value);
     Py_END_ALLOW_THREADS
     if (!result) {
@@ -520,8 +800,8 @@ value. If there is no corresponding record, a new record is stored.");
 static PyObject *
 RTDB_putcat(RTDB *self, PyObject *args, PyObject *kwargs)
 {
-    char *key;
-    Py_ssize_t key_size;
+    void *key;
+    int key_size;
     TCMAP *value;
     PyObject *pykey, *pyvalue = NULL;
     bool result;
@@ -535,7 +815,7 @@ RTDB_putcat(RTDB *self, PyObject *args, PyObject *kwargs)
     if (!pyvalue) {
         return NULL;
     }
-    if(PyBytes_AsStringAndSize(pykey, &key, &key_size)) {
+    if (bytes_to_void(pykey, &key, &key_size)) {
         return NULL;
     }
     value = dict_to_tcmap(pyvalue);
@@ -543,7 +823,7 @@ RTDB_putcat(RTDB *self, PyObject *args, PyObject *kwargs)
         return NULL;
     }
     Py_BEGIN_ALLOW_THREADS
-    result = tcrdbtblputcat(rdbbase->rdb, key, (int)key_size, value);
+    result = tcrdbtblputcat(rdbbase->rdb, key, key_size, value);
     tcmapdel(value);
     Py_END_ALLOW_THREADS
     if (!result) {
@@ -645,6 +925,10 @@ RTDB_metasearch(RTDB *notused, PyObject *args)
         return NULL;
     }
     len = PySequence_Fast_GET_SIZE(pyseq);
+    if (check_py_ssize_t_len(len, pyseq)) {
+        Py_DECREF(pyseq);
+        return NULL;
+    }
     queries = (RDBQRY **)tcmalloc((size_t)len * sizeof(RDBQRY *));
     if (!queries) {
         Py_DECREF(pyseq);
@@ -670,6 +954,58 @@ RTDB_metasearch(RTDB *notused, PyObject *args)
 }
 
 
+/* RTDB.itervalues() */
+PyDoc_STRVAR(RTDB_itervalues_doc,
+"itervalues()\n\
+\n\
+Return an iterator over the database's values.");
+
+static PyObject *
+RTDB_itervalues(RTDB *self)
+{
+    return new_RDBBaseIter((RDBBase *)self, &RTDBIterValuesType);
+}
+
+
+/* RTDB.iteritems() */
+PyDoc_STRVAR(RTDB_iteritems_doc,
+"iteritems()\n\
+\n\
+Return an iterator over the database's items.");
+
+static PyObject *
+RTDB_iteritems(RTDB *self)
+{
+    return new_RDBBaseIter((RDBBase *)self, &RTDBIterItemsType);
+}
+
+
+/* RTDB.itervalueskeys() */
+PyDoc_STRVAR(RTDB_itervalueskeys_doc,
+"itervalueskeys()\n\
+\n\
+Return an iterator over the database's values' keys.");
+
+static PyObject *
+RTDB_itervalueskeys(RTDB *self)
+{
+    return new_RDBBaseIter((RDBBase *)self, &RTDBIterValuesKeysType);
+}
+
+
+/* RTDB.itervaluesvals() */
+PyDoc_STRVAR(RTDB_itervaluesvals_doc,
+"itervaluesvals()\n\
+\n\
+Return an iterator over the database's values' values.");
+
+static PyObject *
+RTDB_itervaluesvals(RTDB *self)
+{
+    return new_RDBBaseIter((RDBBase *)self, &RTDBIterValuesValsType);
+}
+
+
 /* RTDBType.tp_methods */
 static PyMethodDef RTDB_tp_methods[] = {
     {"open", (PyCFunction)RTDB_open, METH_VARARGS | METH_KEYWORDS, RTDB_open_doc},
@@ -685,6 +1021,13 @@ static PyMethodDef RTDB_tp_methods[] = {
     {"query", (PyCFunction)RTDB_query, METH_NOARGS, RTDB_query_doc},
     {"metasearch", (PyCFunction)RTDB_metasearch, METH_VARARGS | METH_STATIC,
      RTDB_metasearch_doc},
+    {"itervalues", (PyCFunction)RTDB_itervalues, METH_NOARGS,
+     RTDB_itervalues_doc},
+    {"iteritems", (PyCFunction)RTDB_iteritems, METH_NOARGS, RTDB_iteritems_doc},
+    {"itervalueskeys", (PyCFunction)RTDB_itervalueskeys, METH_NOARGS,
+     RTDB_itervalueskeys_doc},
+    {"itervaluesvals", (PyCFunction)RTDB_itervaluesvals, METH_NOARGS,
+     RTDB_itervaluesvals_doc},
     {NULL}  /* Sentinel */
 };
 
