@@ -15,6 +15,34 @@ set_tdb_error(TCTDB *tdb, const char *key)
 }
 
 
+/* same as tctdbiternext3 without adding primary key */
+TCMAP *pytctdbiternext3(TCTDB *tdb){
+    assert(tdb);
+    TCXSTR *kstr = tcxstrnew();
+    TCXSTR *vstr = tcxstrnew();
+    TCMAP *cols = NULL;
+    if(tchdbiternext3(tdb->hdb, kstr, vstr)){
+        cols = tcmapload(TCXSTRPTR(vstr), TCXSTRSIZE(vstr));
+    }
+    tcxstrdel(vstr);
+    tcxstrdel(kstr);
+    return cols;
+}
+
+
+/* same as above but keep the key */
+TCMAP *pytctdbiternext4(TCTDB *tdb, TCXSTR *kstr){
+    assert(tdb && kstr);
+    TCXSTR *vstr = tcxstrnew();
+    TCMAP *cols = NULL;
+    if(tchdbiternext3(tdb->hdb, kstr, vstr)){
+        cols = tcmapload(TCXSTRPTR(vstr), TCXSTRSIZE(vstr));
+    }
+    tcxstrdel(vstr);
+    return cols;
+}
+
+
 /*******************************************************************************
 * TDBQueryType
 *******************************************************************************/
@@ -28,7 +56,7 @@ TDBQuery_process_cb(const void *key, int key_size, TCMAP *value, void *op)
     TCMAP *new_value;
     TCMAP tmp_value;
 
-    pykey = PyBytes_FromStringAndSize(key, (Py_ssize_t)key_size);
+    pykey = void_to_bytes(key, key_size);
     if (!pykey) {
         goto fail;
     }
@@ -41,9 +69,9 @@ TDBQuery_process_cb(const void *key, int key_size, TCMAP *value, void *op)
         goto fail;
     }
 #if PY_MAJOR_VERSION >= 3
-    if(PyLong_Check(pyresult)) {
+    if (PyLong_Check(pyresult)) {
 #else
-    if(PyInt_Check(pyresult)) {
+    if (PyInt_Check(pyresult)) {
 #endif
         result = (int)PyLong_AsLong(pyresult);
         if (result == -1 && PyErr_Occurred()) {
@@ -181,6 +209,7 @@ static PyObject *
 TDBQuery_process(TDBQuery *self, PyObject *args)
 {
     PyObject *callback;
+    bool result;
 
     if (!PyArg_ParseTuple(args, "O:process", &callback)) {
         return NULL;
@@ -188,7 +217,8 @@ TDBQuery_process(TDBQuery *self, PyObject *args)
     if (!PyCallable_Check(callback)) {
         return set_error(PyExc_TypeError, "a callable is required");
     }
-    if (!tctdbqryproc(self->qry, TDBQuery_process_cb, (void *)callback)) {
+    result = tctdbqryproc(self->qry, TDBQuery_process_cb, (void *)callback);
+    if (!result) {
         return set_tdb_error(self->tdb->tdb, NULL);
     }
     if (PyErr_Occurred()) {
@@ -267,6 +297,17 @@ TDBQuery_filter(TDBQuery *self, PyObject *args)
 }
 
 
+/* TDBQuery.count() */
+PyDoc_STRVAR(TDBQuery_count_doc,
+"Returns the length of the result set.");
+
+static PyObject *
+TDBQuery_count(TDBQuery *self)
+{
+    return PyInt_FromLong((long)tctdbqrycount(self->qry));
+}
+
+
 /* TDBQueryType.tp_methods */
 static PyMethodDef TDBQuery_tp_methods[] = {
     {"search", (PyCFunction)TDBQuery_search, METH_NOARGS, TDBQuery_search_doc},
@@ -276,6 +317,7 @@ static PyMethodDef TDBQuery_tp_methods[] = {
     {"sort", (PyCFunction)TDBQuery_sort, METH_VARARGS, TDBQuery_sort_doc},
     {"limit", (PyCFunction)TDBQuery_limit, METH_VARARGS, TDBQuery_limit_doc},
     {"filter", (PyCFunction)TDBQuery_filter, METH_VARARGS, TDBQuery_filter_doc},
+    {"count", (PyCFunction)TDBQuery_count, METH_NOARGS, TDBQuery_count_doc},
     {NULL}  /* Sentinel */
 };
 
@@ -287,7 +329,7 @@ PyDoc_STRVAR(TDBQuery_hint_doc,
 static PyObject *
 TDBQuery_hint_get(TDBQuery *self, void *closure)
 {
-    return Py_BuildValue("s", tctdbqryhint(self->qry));
+    return PyString_FromString(tctdbqryhint(self->qry));
 }
 
 
@@ -333,6 +375,332 @@ static PyTypeObject TDBQueryType = {
 };
 
 
+/*******************************************************************************
+* TDB iterator types
+*******************************************************************************/
+
+/* new_TDBIter */
+static PyObject *
+new_TDBIter(TDB *self, PyTypeObject *type)
+{
+    PyObject *iter = DBIter_tp_new(type, (PyObject *)self);
+    if (!iter) {
+        return NULL;
+    }
+    if (!tctdbiterinit(self->tdb)) {
+        Py_DECREF(iter);
+        return set_tdb_error(self->tdb, NULL);
+    }
+    self->changed = false;
+    return iter;
+}
+
+
+/* TDBIterKeysType.tp_iternext */
+static PyObject *
+TDBIterKeys_tp_iternext(DBIter *self)
+{
+    TDB *tdb = (TDB *)self->db;
+    void *key;
+    int key_size;
+    PyObject *pykey;
+
+    if (tdb->changed) {
+        return set_error(Error, "TDB changed during iteration");
+    }
+    key = tctdbiternext(tdb->tdb, &key_size);
+    if (!key) {
+        if (tctdbecode(tdb->tdb) == TCENOREC) {
+            return set_stopiteration_error();
+        }
+        return set_tdb_error(tdb->tdb, NULL);
+    }
+    pykey = void_to_bytes(key, key_size);
+    tcfree(key);
+    return pykey;
+}
+
+
+/* TDBIterKeysType */
+static PyTypeObject TDBIterKeysType = {
+    PyVarObject_HEAD_INIT(NULL, 0)
+    "tokyo.cabinet.TDBIterKeys",              /*tp_name*/
+    sizeof(DBIter),                           /*tp_basicsize*/
+    0,                                        /*tp_itemsize*/
+    (destructor)DBIter_tp_dealloc,            /*tp_dealloc*/
+    0,                                        /*tp_print*/
+    0,                                        /*tp_getattr*/
+    0,                                        /*tp_setattr*/
+    0,                                        /*tp_compare*/
+    0,                                        /*tp_repr*/
+    0,                                        /*tp_as_number*/
+    0,                                        /*tp_as_sequence*/
+    0,                                        /*tp_as_mapping*/
+    0,                                        /*tp_hash */
+    0,                                        /*tp_call*/
+    0,                                        /*tp_str*/
+    0,                                        /*tp_getattro*/
+    0,                                        /*tp_setattro*/
+    0,                                        /*tp_as_buffer*/
+    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_HAVE_GC,  /*tp_flags*/
+    0,                                        /*tp_doc*/
+    (traverseproc)DBIter_tp_traverse,         /*tp_traverse*/
+    (inquiry)DBIter_tp_clear,                 /*tp_clear*/
+    0,                                        /*tp_richcompare*/
+    0,                                        /*tp_weaklistoffset*/
+    PyObject_SelfIter,                        /*tp_iter*/
+    (iternextfunc)TDBIterKeys_tp_iternext,    /*tp_iternext*/
+    DBIter_tp_methods,                        /*tp_methods*/
+};
+
+
+/* TDBIterValuesType.tp_iternext */
+static PyObject *
+TDBIterValues_tp_iternext(DBIter *self)
+{
+    TDB *tdb = (TDB *)self->db;
+    TCMAP *value;
+    PyObject *pyvalue;
+
+    if (tdb->changed) {
+        return set_error(Error, "TDB changed during iteration");
+    }
+    value = pytctdbiternext3(tdb->tdb);
+    if (!value) {
+        if (tctdbecode(tdb->tdb) == TCENOREC) {
+            return set_stopiteration_error();
+        }
+        return set_tdb_error(tdb->tdb, NULL);
+    }
+    pyvalue = tcmap_to_dict(value);
+    tcmapdel(value);
+    return pyvalue;
+}
+
+
+/* TDBIterValuesType */
+static PyTypeObject TDBIterValuesType = {
+    PyVarObject_HEAD_INIT(NULL, 0)
+    "tokyo.cabinet.TDBIterValues",            /*tp_name*/
+    sizeof(DBIter),                           /*tp_basicsize*/
+    0,                                        /*tp_itemsize*/
+    (destructor)DBIter_tp_dealloc,            /*tp_dealloc*/
+    0,                                        /*tp_print*/
+    0,                                        /*tp_getattr*/
+    0,                                        /*tp_setattr*/
+    0,                                        /*tp_compare*/
+    0,                                        /*tp_repr*/
+    0,                                        /*tp_as_number*/
+    0,                                        /*tp_as_sequence*/
+    0,                                        /*tp_as_mapping*/
+    0,                                        /*tp_hash */
+    0,                                        /*tp_call*/
+    0,                                        /*tp_str*/
+    0,                                        /*tp_getattro*/
+    0,                                        /*tp_setattro*/
+    0,                                        /*tp_as_buffer*/
+    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_HAVE_GC,  /*tp_flags*/
+    0,                                        /*tp_doc*/
+    (traverseproc)DBIter_tp_traverse,         /*tp_traverse*/
+    (inquiry)DBIter_tp_clear,                 /*tp_clear*/
+    0,                                        /*tp_richcompare*/
+    0,                                        /*tp_weaklistoffset*/
+    PyObject_SelfIter,                        /*tp_iter*/
+    (iternextfunc)TDBIterValues_tp_iternext,  /*tp_iternext*/
+    DBIter_tp_methods,                        /*tp_methods*/
+};
+
+
+/* TDBIterItemsType.tp_iternext */
+static PyObject *
+TDBIterItems_tp_iternext(DBIter *self)
+{
+    TDB *tdb = (TDB *)self->db;
+    TCXSTR *key;
+    TCMAP *value;
+    PyObject *pykey, *pyvalue, *pyresult = NULL;
+
+    if (tdb->changed) {
+        return set_error(Error, "TDB changed during iteration");
+    }
+    key = tcxstrnew();
+    value = pytctdbiternext4(tdb->tdb, key);
+    if (!value) {
+        if (tctdbecode(tdb->tdb) == TCENOREC) {
+            set_stopiteration_error();
+        }
+        else {
+            set_tdb_error(tdb->tdb, NULL);
+        }
+    }
+    else {
+        pykey = tcxstr_to_bytes(key);
+        pyvalue = tcmap_to_dict(value);
+        if (pykey && pyvalue) {
+            pyresult = PyTuple_Pack(2, pykey, pyvalue);
+        }
+        tcmapdel(value);
+        Py_XDECREF(pykey);
+        Py_XDECREF(pyvalue);
+    }
+    tcxstrdel(key);
+    return pyresult;
+}
+
+
+/* TDBIterItemsType */
+static PyTypeObject TDBIterItemsType = {
+    PyVarObject_HEAD_INIT(NULL, 0)
+    "tokyo.cabinet.TDBIterItems",             /*tp_name*/
+    sizeof(DBIter),                           /*tp_basicsize*/
+    0,                                        /*tp_itemsize*/
+    (destructor)DBIter_tp_dealloc,            /*tp_dealloc*/
+    0,                                        /*tp_print*/
+    0,                                        /*tp_getattr*/
+    0,                                        /*tp_setattr*/
+    0,                                        /*tp_compare*/
+    0,                                        /*tp_repr*/
+    0,                                        /*tp_as_number*/
+    0,                                        /*tp_as_sequence*/
+    0,                                        /*tp_as_mapping*/
+    0,                                        /*tp_hash */
+    0,                                        /*tp_call*/
+    0,                                        /*tp_str*/
+    0,                                        /*tp_getattro*/
+    0,                                        /*tp_setattro*/
+    0,                                        /*tp_as_buffer*/
+    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_HAVE_GC,  /*tp_flags*/
+    0,                                        /*tp_doc*/
+    (traverseproc)DBIter_tp_traverse,         /*tp_traverse*/
+    (inquiry)DBIter_tp_clear,                 /*tp_clear*/
+    0,                                        /*tp_richcompare*/
+    0,                                        /*tp_weaklistoffset*/
+    PyObject_SelfIter,                        /*tp_iter*/
+    (iternextfunc)TDBIterItems_tp_iternext,   /*tp_iternext*/
+    DBIter_tp_methods,                        /*tp_methods*/
+};
+
+
+/* TDBIterValuesKeysType.tp_iternext */
+static PyObject *
+TDBIterValuesKeys_tp_iternext(DBIter *self)
+{
+    TDB *tdb = (TDB *)self->db;
+    TCMAP *value;
+    TCLIST *valuekeys;
+    PyObject *pyvaluekeys;
+
+    if (tdb->changed) {
+        return set_error(Error, "TDB changed during iteration");
+    }
+    value = pytctdbiternext3(tdb->tdb);
+    if (!value) {
+        if (tctdbecode(tdb->tdb) == TCENOREC) {
+            return set_stopiteration_error();
+        }
+        return set_tdb_error(tdb->tdb, NULL);
+    }
+    valuekeys = tcmapkeys(value);
+    pyvaluekeys = tclist_to_tuple(valuekeys);
+    tcmapdel(value);
+    tclistdel(valuekeys);
+    return pyvaluekeys;
+}
+
+
+/* TDBIterValuesKeysType */
+static PyTypeObject TDBIterValuesKeysType = {
+    PyVarObject_HEAD_INIT(NULL, 0)
+    "tokyo.cabinet.TDBIterValuesKeys",        /*tp_name*/
+    sizeof(DBIter),                           /*tp_basicsize*/
+    0,                                        /*tp_itemsize*/
+    (destructor)DBIter_tp_dealloc,            /*tp_dealloc*/
+    0,                                        /*tp_print*/
+    0,                                        /*tp_getattr*/
+    0,                                        /*tp_setattr*/
+    0,                                        /*tp_compare*/
+    0,                                        /*tp_repr*/
+    0,                                        /*tp_as_number*/
+    0,                                        /*tp_as_sequence*/
+    0,                                        /*tp_as_mapping*/
+    0,                                        /*tp_hash */
+    0,                                        /*tp_call*/
+    0,                                        /*tp_str*/
+    0,                                        /*tp_getattro*/
+    0,                                        /*tp_setattro*/
+    0,                                        /*tp_as_buffer*/
+    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_HAVE_GC,  /*tp_flags*/
+    0,                                        /*tp_doc*/
+    (traverseproc)DBIter_tp_traverse,         /*tp_traverse*/
+    (inquiry)DBIter_tp_clear,                 /*tp_clear*/
+    0,                                        /*tp_richcompare*/
+    0,                                        /*tp_weaklistoffset*/
+    PyObject_SelfIter,                        /*tp_iter*/
+    (iternextfunc)TDBIterValuesKeys_tp_iternext, /*tp_iternext*/
+    DBIter_tp_methods,                        /*tp_methods*/
+};
+
+
+/* TDBIterValuesValsType.tp_iternext */
+static PyObject *
+TDBIterValuesVals_tp_iternext(DBIter *self)
+{
+    TDB *tdb = (TDB *)self->db;
+    TCMAP *value;
+    TCLIST *valuevals;
+    PyObject *pyvaluevals;
+
+    if (tdb->changed) {
+        return set_error(Error, "TDB changed during iteration");
+    }
+    value = pytctdbiternext3(tdb->tdb);
+    if (!value) {
+        if (tctdbecode(tdb->tdb) == TCENOREC) {
+            return set_stopiteration_error();
+        }
+        return set_tdb_error(tdb->tdb, NULL);
+    }
+    valuevals = tcmapvals(value);
+    pyvaluevals = tclist_to_tuple(valuevals);
+    tcmapdel(value);
+    tclistdel(valuevals);
+    return pyvaluevals;
+}
+
+
+/* TDBIterValuesValsType */
+static PyTypeObject TDBIterValuesValsType = {
+    PyVarObject_HEAD_INIT(NULL, 0)
+    "tokyo.cabinet.TDBIterValuesVals",        /*tp_name*/
+    sizeof(DBIter),                           /*tp_basicsize*/
+    0,                                        /*tp_itemsize*/
+    (destructor)DBIter_tp_dealloc,            /*tp_dealloc*/
+    0,                                        /*tp_print*/
+    0,                                        /*tp_getattr*/
+    0,                                        /*tp_setattr*/
+    0,                                        /*tp_compare*/
+    0,                                        /*tp_repr*/
+    0,                                        /*tp_as_number*/
+    0,                                        /*tp_as_sequence*/
+    0,                                        /*tp_as_mapping*/
+    0,                                        /*tp_hash */
+    0,                                        /*tp_call*/
+    0,                                        /*tp_str*/
+    0,                                        /*tp_getattro*/
+    0,                                        /*tp_setattro*/
+    0,                                        /*tp_as_buffer*/
+    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_HAVE_GC,  /*tp_flags*/
+    0,                                        /*tp_doc*/
+    (traverseproc)DBIter_tp_traverse,         /*tp_traverse*/
+    (inquiry)DBIter_tp_clear,                 /*tp_clear*/
+    0,                                        /*tp_richcompare*/
+    0,                                        /*tp_weaklistoffset*/
+    PyObject_SelfIter,                        /*tp_iter*/
+    (iternextfunc)TDBIterValuesVals_tp_iternext, /*tp_iternext*/
+    DBIter_tp_methods,                        /*tp_methods*/
+};
+
+
 /*******************************************************************************
 * TDBType
 *******************************************************************************/
@@ -387,14 +755,14 @@ TDB_tp_new(PyTypeObject *type, PyObject *args, PyObject *kwargs)
 static
 int TDB_Contains(TDB *self, PyObject *pykey)
 {
-    char *key;
-    Py_ssize_t key_size;
+    void *key;
+    int key_size;
     TCMAP *value;
 
-    if(PyBytes_AsStringAndSize(pykey, &key, &key_size)) {
+    if (bytes_to_void(pykey, &key, &key_size)) {
         return -1;
     }
-    value = tctdbget(self->tdb, key, (int)key_size);
+    value = tctdbget(self->tdb, key, key_size);
     if (!value) {
         if (tctdbecode(self->tdb) == TCENOREC) {
             return 0;
@@ -424,7 +792,7 @@ static PySequenceMethods TDB_tp_as_sequence = {
 static Py_ssize_t
 TDB_Length(TDB *self)
 {
-    return (Py_ssize_t)tctdbrnum(self->tdb);
+    return DB_Length(tctdbrnum(self->tdb));
 }
 
 
@@ -432,15 +800,15 @@ TDB_Length(TDB *self)
 static PyObject *
 TDB_GetItem(TDB *self, PyObject *pykey)
 {
-    char *key;
-    Py_ssize_t key_size;
+    void *key;
+    int key_size;
     TCMAP *value;
     PyObject *pyvalue;
 
-    if(PyBytes_AsStringAndSize(pykey, &key, &key_size)) {
+    if (bytes_to_void(pykey, &key, &key_size)) {
         return NULL;
     }
-    value = tctdbget(self->tdb, key, (int)key_size);
+    value = tctdbget(self->tdb, key, key_size);
     if (!value) {
         return set_tdb_error(self->tdb, key);
     }
@@ -454,11 +822,11 @@ TDB_GetItem(TDB *self, PyObject *pykey)
 static int
 TDB_SetItem(TDB *self, PyObject *pykey, PyObject *pyvalue)
 {
-    char *key;
-    Py_ssize_t key_size;
+    void *key;
+    int key_size;
     TCMAP *value;
 
-    if(PyBytes_AsStringAndSize(pykey, &key, &key_size)) {
+    if (bytes_to_void(pykey, &key, &key_size)) {
         return -1;
     }
     if (pyvalue) {
@@ -466,7 +834,7 @@ TDB_SetItem(TDB *self, PyObject *pykey, PyObject *pyvalue)
         if (!value) {
             return -1;
         }
-        if (!tctdbput(self->tdb, key, (int)key_size, value)) {
+        if (!tctdbput(self->tdb, key, key_size, value)) {
             tcmapdel(value);
             set_tdb_error(self->tdb, NULL);
             return -1;
@@ -474,7 +842,7 @@ TDB_SetItem(TDB *self, PyObject *pykey, PyObject *pyvalue)
         tcmapdel(value);
     }
     else {
-        if (!tctdbout(self->tdb, key, (int)key_size)) {
+        if (!tctdbout(self->tdb, key, key_size)) {
             set_tdb_error(self->tdb, key);
             return -1;
         }
@@ -496,46 +864,7 @@ static PyMappingMethods TDB_tp_as_mapping = {
 static PyObject *
 TDB_tp_iter(TDB *self)
 {
-    if (!tctdbiterinit(self->tdb)) {
-        return set_tdb_error(self->tdb, NULL);
-    }
-    self->changed = false;
-    Py_INCREF(self);
-    return (PyObject *)self;
-}
-
-
-/* TDBType.tp_iternext */
-static PyObject *
-TDB_tp_iternext(TDB *self)
-{
-    const char *key;
-    PyObject *pykey;
-
-    if (self->changed) {
-        return set_error(Error, "TDB changed during iteration");
-    }
-    key = tctdbiternext2(self->tdb);
-    if (!key) {
-        if (tctdbecode(self->tdb) == TCENOREC) {
-            return set_stopiteration_error();
-        }
-        return set_tdb_error(self->tdb, NULL);
-    }
-    pykey = PyBytes_FromString(key);
-    tcfree((void *)key);
-    return pykey;
-}
-
-
-/* TDB.__length_hint__ */
-PyDoc_STRVAR(TDB_length_hint_doc,
-"Private method returning an estimate of len(list(tdb)).");
-
-static PyObject *
-TDB_length_hint(TDB *self)
-{
-    return PyLong_FromSsize_t(TDB_Length(self));
+    return new_TDBIter(self, &TDBIterKeysType);
 }
 
 
@@ -749,8 +1078,8 @@ put), this method raises KeyError if key is already in the database.");
 static PyObject *
 TDB_putkeep(TDB *self, PyObject *args, PyObject *kwargs)
 {
-    char *key;
-    Py_ssize_t key_size;
+    void *key;
+    int key_size;
     TCMAP *value;
     PyObject *pykey, *pyvalue = NULL;
 
@@ -762,14 +1091,14 @@ TDB_putkeep(TDB *self, PyObject *args, PyObject *kwargs)
     if (!pyvalue) {
         return NULL;
     }
-    if(PyBytes_AsStringAndSize(pykey, &key, &key_size)) {
+    if (bytes_to_void(pykey, &key, &key_size)) {
         return NULL;
     }
     value = dict_to_tcmap(pyvalue);
     if (!value) {
         return NULL;
     }
-    if (!tctdbputkeep(self->tdb, key, (int)key_size, value)) {
+    if (!tctdbputkeep(self->tdb, key, key_size, value)) {
         tcmapdel(value);
         return set_tdb_error(self->tdb, key);
     }
@@ -789,8 +1118,8 @@ value. If there is no corresponding record, a new record is stored.");
 static PyObject *
 TDB_putcat(TDB *self, PyObject *args, PyObject *kwargs)
 {
-    char *key;
-    Py_ssize_t key_size;
+    void *key;
+    int key_size;
     TCMAP *value;
     PyObject *pykey, *pyvalue = NULL;
 
@@ -802,14 +1131,14 @@ TDB_putcat(TDB *self, PyObject *args, PyObject *kwargs)
     if (!pyvalue) {
         return NULL;
     }
-    if(PyBytes_AsStringAndSize(pykey, &key, &key_size)) {
+    if (bytes_to_void(pykey, &key, &key_size)) {
         return NULL;
     }
     value = dict_to_tcmap(pyvalue);
     if (!value) {
         return NULL;
     }
-    if (!tctdbputcat(self->tdb, key, (int)key_size, value)) {
+    if (!tctdbputcat(self->tdb, key, key_size, value)) {
         tcmapdel(value);
         return set_tdb_error(self->tdb, NULL);
     }
@@ -846,20 +1175,19 @@ is applied.");
 static PyObject *
 TDB_searchkeys(TDB *self, PyObject *args)
 {
-    const char *prefix;
-    int max = -1;
+    void *prefix;
+    int prefix_size, max = -1;
     TCLIST *result;
     PyObject *pyprefix, *pyresult;
 
     if (!PyArg_ParseTuple(args, "O|i:searchkeys", &pyprefix, &max)) {
         return NULL;
     }
-    prefix = PyBytes_AsString(pyprefix);
-    if (!prefix) {
+    if (bytes_to_void(pyprefix, &prefix, &prefix_size)) {
         return NULL;
     }
     Py_BEGIN_ALLOW_THREADS
-    result = tctdbfwmkeys2(self->tdb, prefix, max);
+    result = tctdbfwmkeys(self->tdb, prefix, prefix_size, max);
     Py_END_ALLOW_THREADS
     pyresult = tclist_to_frozenset(result);
     tclistdel(result);
@@ -1110,6 +1438,10 @@ TDB_metasearch(TDB *notused, PyObject *args)
         return NULL;
     }
     len = PySequence_Fast_GET_SIZE(pyseq);
+    if (check_py_ssize_t_len(len, pyseq)) {
+        Py_DECREF(pyseq);
+        return NULL;
+    }
     queries = (TDBQRY **)tcmalloc((size_t)len * sizeof(TDBQRY *));
     if (!queries) {
         Py_DECREF(pyseq);
@@ -1135,10 +1467,73 @@ TDB_metasearch(TDB *notused, PyObject *args)
 }
 
 
+/* TDB.iterkeys() */
+PyDoc_STRVAR(TDB_iterkeys_doc,
+"iterkeys()\n\
+\n\
+Return an iterator over the database's keys.");
+
+static PyObject *
+TDB_iterkeys(TDB *self)
+{
+    return new_TDBIter(self, &TDBIterKeysType);
+}
+
+
+/* TDB.itervalues() */
+PyDoc_STRVAR(TDB_itervalues_doc,
+"itervalues()\n\
+\n\
+Return an iterator over the database's values.");
+
+static PyObject *
+TDB_itervalues(TDB *self)
+{
+    return new_TDBIter(self, &TDBIterValuesType);
+}
+
+
+/* TDB.iteritems() */
+PyDoc_STRVAR(TDB_iteritems_doc,
+"iteritems()\n\
+\n\
+Return an iterator over the database's items.");
+
+static PyObject *
+TDB_iteritems(TDB *self)
+{
+    return new_TDBIter(self, &TDBIterItemsType);
+}
+
+
+/* TDB.itervalueskeys() */
+PyDoc_STRVAR(TDB_itervalueskeys_doc,
+"itervalueskeys()\n\
+\n\
+Return an iterator over the database's values' keys.");
+
+static PyObject *
+TDB_itervalueskeys(TDB *self)
+{
+    return new_TDBIter(self, &TDBIterValuesKeysType);
+}
+
+
+/* TDB.itervaluesvals() */
+PyDoc_STRVAR(TDB_itervaluesvals_doc,
+"itervaluesvals()\n\
+\n\
+Return an iterator over the database's values' values.");
+
+static PyObject *
+TDB_itervaluesvals(TDB *self)
+{
+    return new_TDBIter(self, &TDBIterValuesValsType);
+}
+
+
 /* TDBType.tp_methods */
 static PyMethodDef TDB_tp_methods[] = {
-    {"__length_hint__", (PyCFunction)TDB_length_hint, METH_NOARGS,
-     TDB_length_hint_doc},
     {"open", (PyCFunction)TDB_open, METH_VARARGS, TDB_open_doc},
     {"close", (PyCFunction)TDB_close, METH_NOARGS, TDB_close_doc},
     {"clear", (PyCFunction)TDB_clear, METH_NOARGS, TDB_clear_doc},
@@ -1167,6 +1562,13 @@ static PyMethodDef TDB_tp_methods[] = {
     {"query", (PyCFunction)TDB_query, METH_NOARGS, TDB_query_doc},
     {"metasearch", (PyCFunction)TDB_metasearch, METH_VARARGS | METH_STATIC,
      TDB_metasearch_doc},
+    {"iterkeys", (PyCFunction)TDB_iterkeys, METH_NOARGS, TDB_iterkeys_doc},
+    {"itervalues", (PyCFunction)TDB_itervalues, METH_NOARGS, TDB_itervalues_doc},
+    {"iteritems", (PyCFunction)TDB_iteritems, METH_NOARGS, TDB_iteritems_doc},
+    {"itervalueskeys", (PyCFunction)TDB_itervalueskeys, METH_NOARGS,
+     TDB_itervalueskeys_doc},
+    {"itervaluesvals", (PyCFunction)TDB_itervaluesvals, METH_NOARGS,
+     TDB_itervaluesvals_doc},
     {NULL}  /* Sentinel */
 };
 
@@ -1182,7 +1584,7 @@ TDB_path_get(TDB *self, void *closure)
 
     path = tctdbpath(self->tdb);
     if (path) {
-        return Py_BuildValue("s", path);
+        return PyString_FromString(path);
     }
     Py_RETURN_NONE;
 }
@@ -1235,7 +1637,7 @@ static PyTypeObject TDBType = {
     0,                                        /*tp_richcompare*/
     0,                                        /*tp_weaklistoffset*/
     (getiterfunc)TDB_tp_iter,                 /*tp_iter*/
-    (iternextfunc)TDB_tp_iternext,            /*tp_iternext*/
+    0,                                        /*tp_iternext*/
     TDB_tp_methods,                           /*tp_methods*/
     0,                                        /*tp_members*/
     TDB_tp_getsets,                           /*tp_getsets*/
